\ Analoge datalogger met uitvoer naar RS232 en LEDS: 698/654 bytes. 

89C2051  TARGET                         \ Compileer voor AT89C2051

\ ByteForth RS232 woorden

$99 BIT-SFR RS232-EMIT?                 \ RS232 zend buffer leeg ?
$8E BIT-SFR TIMER1                      \ Timer 1 start/stop
$AC BIT-SFR RS232-INTERRUPT             \ Timer 1 interrupt aan/uit
ASSEMBLER                               \ Gebruik assembler labels
TMOD: SFR TIMER-MODE                    \ Timer instelling
PCON: SFR POWER-MODE                    \ RS232 snelheid verdubbeling
SCON: SFR RS232-MODE                    \ RS232 instelling
TH1:  SFR TIMER1-HIGH                   \ Hoge byte van timer 1
SBUF: SFR RS232-BUFFER                  \ RS232 in/uitvoer buffer
WORK

: RS232-SETUP
    TIMER-MODE  $20 OR  TO TIMER-MODE   \ Timer-1 mode 2 = 8 bit autoreload
    243 TO TIMER1-HIGH                  \ Selecteer 1200/2400 baud
    SET TIMER1                          \ Start timer 1
    POWER-MODE  $80 OR  TO POWER-MODE   \ Verdubbel RS232 ( 2400 baud )
    $52 TO RS232-MODE                   \ Seriele mode 1, ontvangst aan
                                        \ Zet zend interrupt vlag
    CLEAR RS232-INTERRUPT               \ Zet RS232 interrupt uit
    ;

: RS232-EMIT        ( char -- )         \ Stuur karakter via RS232
    BEGIN  RS232-EMIT? UNTIL
    TO RS232-BUFFER
    CLEAR RS232-EMIT?
    ;

: RS232-TYPE        ( $addr u -- )      \ Stuur ROM string via RS232
    ?DUP IF
        FOR
            COUNT RS232-EMIT
        NEXT
    THEN
    2DROP
    ;

: RS232-RTYPE       ( addr u -- )       \ Stuur RAM string via RS232
    ?DUP IF
        FOR
            @+ RS232-EMIT
        NEXT
    THEN
    DROP
    ;

: TYPE      RS232-TYPE ;                        ( $addr u -- )
: RTYPE     RS232-RTYPE ;                       ( addr u -- )
: CR        ^M RS232-EMIT  ^J RS232-EMIT ;      ( -- )
: SPACE     BL RS232-EMIT ;                     ( -- )
: PAGE      ^L RS232-EMIT ;                     ( -- )

\ Einde RS232

\ Getal uitvoer

ATOM UM/MOD                             \ Importeer enkele macros als woord
ATOM INLINE$
VARIABLE BASE                           \ Getal conversie basis
VARIABLE HLD                            \ Werk adr. in getal conv. buffer
$80 CONSTANT NUM                        \ Start adr. van getal conversie buffer
                                        \ Alleen geldig voor de AT89C2051

: M/MOD     ( d1 u1 -- u2 d2 )          \ Deel dubbel met teken door single
    >R  0  R@ UM/MOD  R> SWAP >R  UM/MOD R> \ zonder teken.
    ;

\ Getal uitvoer primitieven

\ Zet karakter char in de uitvoer buffer
: HOLD      DECR HLD  FROM HLD ! ;                      ( char -- )

\ Initialiseer de uitvoer buffer
: <#        NUM TO HLD ;                                ( -- )

\ Beeindig getal conversie, geef addr en lengte u van de string in de buffer
: #>        2DROP  FROM HLD  NUM FROM HLD - ;           ( d -- addr u )

\ Zet een digit van d1 in de uitvoer buffer, d2 blijft over
: #         FROM BASE M/MOD  ROT 9 OVER <               ( d1 -- d2 )
            IF  7 +  THEN  [CHAR] 0 +  HOLD ;

\ Getal uitvoer woorden

\ Druk getal u af met digits en twee spaties er achter
: .DATA     0 <# # # # #>  RTYPE SPACE SPACE ;          ( u -- )

\ Zet getal conversie basis decimaal
: DECIMAL   #10 TO BASE ;                               ( -- )

\ Einde getal uitvoer

\ Data logger die elke seconde een meting bewaard en na 20 metingen
\ de data in een twee blokken via de RS232 verstuurt.

$90 SFR LEDS                            \ Poort 1 met 8 leds

$B4 BIT-SFR ADC-KLOK                    \ Klokpuls invoer P3.4
$B5 BIT-SFR ADC-DATA                    \ Data uitvoer P3.5
$B7 BIT-SFR ADC-SELECTEER               \ Activeer invoer P3.7

: ADC               ( -- u )            \ Doe een AD-omzetting
    CLEAR ADC-KLOK                      \ Stop lezen van analoge invoer
    SET ADC-SELECTEER                   \ Start AD-omzetting
    4 FOR NEXT                          \ Conversie duurt min. 17 us 
    CLEAR ADC-SELECTEER                 \ Start uitlezen
    ADC-DATA 1 AND                      \ Lees databit 7
    7 FOR                               \ Lees resterende 7 bits
        SET ADC-KLOK
        CLEAR ADC-KLOK                  \ Schuif volgend bit naar buiten
        2*  ADC-DATA  1 AND  OR         \ Schaal data in, lees volgend bit
    NEXT
    SET ADC-KLOK                        \ Blijf analoge invoer lezen
    ;

: 100MS  FOR  100 MS  NEXT ;            \ Wacht u maal 100 milliseconden

: NAAR-LEDS     ( x -- )                \ Zet x op leds
    INVERT  TO LEDS                     \ Inverteer x en stuur leds aan
    ;

: KNIPPER       ( -- )                  \ Visualiseer opstarten
    -1 NAAR-LEDS  250 MS                \ Alle leds aan
    0 NAAR-LEDS   250 MS                \ Alle leds uit
    ;

: BEELD-AF      ( u -- )                \ Toon met VU- of thermometer methode
    0 32 UM/MOD                         \ Schaal waarde in voor 8 leds
    SWAP 15 > IF  1+  THEN              \ Rond af naar hogere waarde
    DUP IF                              \ Uitkomst groter dan nul ?
        1 SWAP 1- LSHIFT                \ Ja, bepaal positie van led
\       DUP 1- OR                       \ Vul leds daaronder in
    THEN
    NAAR-LEDS                           \ Stuur leds aan
    ;

: SAMPLE-ADC        ( -- u )            \ Elke tiende AD-omzetting op de stack
    0                                   \ Dummy 
    10 0 DO                             \ Toon tien metingen op leds
        DROP                            \ Verwijder oude AD-omzetting
        ADC  DUP BEELD-AF               \ Lees ADC, en toon op LEDS
        1 100MS                         \ Wacht 1/10 sec.
    LOOP
    ;

20 VARIABLES DATA                       \ Ruimte voor 20 metingen

: ZEND-TIEN-METINGEN    ( offset -- )   \ Stuur 10 metingen over
    10 0 DO
        DUP I + FROM DATA .DATA
    LOOP
    DROP
    ;

: DATALOGGER    ( -- )                  \ Een eenkanaal analoge datalogger
    SETUP  RS232-SETUP
    DECIMAL  PAGE                       \ Scherm schoon
    ." ADC naar RS232 data logger"      \ Opstart melding
    KNIPPER 
    ADC DROP                            \ Eerste AD-omzetting is ongeldig
    BEGIN
        CR  20 0 DO                     \ Bewaar 20 samples
            SAMPLE-ADC                  \ Sample ADC en toon op LEDS
            I TO DATA                   \ Sla meting I op
        LOOP
        CR 00 ZEND-TIEN-METINGEN        \ Eerste tien
        CR 10 ZEND-TIEN-METINGEN        \ Tweede tien
    AGAIN
    ;

' DATALOGGER  RESET-VEC  SET-VECTOR     \ DATALOGGER start op na een reset

\ Einde
