\ Puls afstand meting (TOERENTELLER, rondentijd, afst-meter etc.) 758/740 bytes.

89C2051  TARGET                         \ Compileer voor AT89C2051

\ RS232 i/o

$99 BIT-SFR RS232-EMIT?                 \ RS232 zend buffer leeg ?
$8E BIT-SFR TIMER1                      \ Timer 1 start/stop
$AC BIT-SFR RS232-INTERRUPT             \ Timer 1 interrupt aan/uit
ASSEMBLER                               \ Gebruik assembler labels
TMOD: SFR TIMER-MODE                    \ Timer instelling
PCON: SFR POWER-MODE                    \ RS232 snelheid verdubbeling
SCON: SFR RS232-MODE                    \ RS232 instelling
TH1:  SFR TIMER1-HIGH                   \ Hoge byte van timer 1
SBUF: SFR RS232-BUFFER                  \ RS232 in/uitvoer buffer
WORK

: RS232-SETUP
    TIMER-MODE  $20 OR  TO TIMER-MODE   \ Timer-1 mode 2 = 8 bit autoreload
    243 TO TIMER1-HIGH                  \ Selecteer 1200/2400 baud
    SET TIMER1                          \ Start timer 1
    POWER-MODE  $80 OR  TO POWER-MODE   \ Verdubbel RS232 ( 2400 baud )
    $52 TO RS232-MODE                   \ Seriele mode 1, ontvangst aan
                                        \ Zet zend interrupt vlag
    CLEAR RS232-INTERRUPT               \ Zet RS232 interrupt uit
    ;

: RS232-EMIT        ( char -- )         \ Een karakter naar RS232
    BEGIN  RS232-EMIT? UNTIL
    TO RS232-BUFFER
    CLEAR RS232-EMIT?
    ;

: RS232-TYPE        ( $addr u -- )      \ Een string uit ROM naar RS232
    ?DUP IF
        FOR
            COUNT RS232-EMIT
        NEXT
    THEN
    2DROP
    ;

: RS232-RTYPE       ( addr u -- )       \ Een string uit RAM naar RS232
    ?DUP IF
        FOR
            @+ RS232-EMIT
        NEXT
    THEN
    DROP
    ;

: TYPE      RS232-TYPE ;                        ( $addr u -- )
: RTYPE     RS232-RTYPE ;                       ( addr u -- )
: CR        ^M RS232-EMIT  ^J RS232-EMIT ;      ( -- )
: SPACE     BL RS232-EMIT ;                     ( -- )
: PAGE      ^L RS232-EMIT ;                     ( -- )

\ Einde RS232
\ Getal uitvoer routines

ATOM UM/MOD                             \ Importeer enkele macros als woord
ATOM INLINE$
VARIABLE BASE                           \ Getal conversie basis
VARIABLE HLD                            \ Werk adr. in getal conv. buffer
$80 CONSTANT NUM                        \ Start adr. van getal conversie buffer
                                        \ Alleen geldig voor de AT89C2051

: M/MOD     ( d1 u1 -- u2 d2 )          \ Deel dubbel met teken door single
    >R  0  R@ UM/MOD  R> SWAP >R  UM/MOD R> \ zonder teken.
    ;

\ Getal uitvoer primitieven

\ Zet karakter char in de uitvoer buffer
: HOLD      DECR HLD  FROM HLD ! ;                      ( char -- )

\ Initialiseer de uitvoer buffer
: <#        NUM TO HLD ;                                ( -- )

\ Beeindig getal conversie, geef addr en lengte u van de string in de buffer
: #>        2DROP  FROM HLD  NUM FROM HLD - ;           ( d -- addr u )

\ Zet een digit van d1 in de uitvoer buffer, d2 blijft over
: #         FROM BASE M/MOD  ROT 9 OVER <               ( d1 -- d2 )
            IF  7 +  THEN  [CHAR] 0 +  HOLD ;

\ Converteer digits van d1 als beschreven bij # tot het resultaat
\ d2 nul is/
: #S        BEGIN  # 2DUP OR 0= UNTIL ;                 ( d1 -- d2 )


\ Getal uitvoer woorden

\ Toon ud in een vrij formaat veld.
: DU.       <# #S #>  RTYPE SPACE ;                     ( d -- )

\G Toon u in een vrij formaat veld.
: U.        0  DU. ;                                    ( u -- )

\ Zet getal conversie basis decimaal
: DECIMAL   #10 TO BASE ;                               ( -- )

\ Einde getal uitvoer
\ Start tijdmeting routines

    $B2 BIT-SFR IN                      \ Periode meet ingang P3.2
    $B5 BIT-SFR STARTKNOP               \ Start meting P3.5

    2VARIABLE TELLER                    \ Periodetijd teller
    2VARIABLE MEETTIJD                  \ Onthoud periodetijd
    2VARIABLE FREQUENTIE                \ Onhoud frequentie

    FLAG DENDER?                        \ Dender onderdrukking
    FLAG METING?                        \ Complete meting gedaan
    FLAG KLAAR?                         \ Start een meetcyclus

\ Interrupt routine voor periodetijd meter, de maximaal te meten
\ periodetijd is: 65535 * 1/1000 sec = 65,5 sec. De kortste tijd
\ in principe 4 millisec. maar deze kan er bijna 20 % naast zitten.
CODE TEL                ( -- )          \ Wordt elke 1/1000 sec geactiveerd
    ACC: PUSH,                          \ Bewaar de gebruikte registers
    PSW: PUSH,
    TH0: -500. NIP # MOV,               \ Zet negative 16 bits waarde in timer 0
    TL0: -500. DROP # MOV,
    ADR TELLER LOW INC,                 \ Verhoog lage byte van teller
    A: ADR TELLER LOW MOV,
    ZER IF,                             \ Overflow ?
        ADR TELLER INC,                 \ Ja, verhoog hoge byte
    THEN,
    A: 4 # EQL IF,                      \ Dender onderdrukking van 4 millisec.
        ADR DENDER? CLR,                \ Sta nieuw signaal toe
    THEN,
    PSW: POP,                           \ Herstel de gebruikte registers
    ACC: POP,
    RETI,
END-CODE

CODE NOTEER             ( -- )          \ Noteer periode tijd
    ADR KLAAR? BS IF,  RETI,  THEN,     \ Meting afgerond? doe dan niets !
    ADR DENDER? BC IF,                  \ Ingang vrij gegeven ?
        ADR METING? BC IF,              \ Begin een meting ?
            ADR TELLER LOW 0 # MOV,     \ Ja, reset teller
            ADR TELLER 0 # MOV,
            ADR METING? SETB,
        ELSE,
            ADR MEETTIJD LOW ADR TELLER LOW MOV, \ Nee, bewaar periodetijd
            ADR MEETTIJD ADR TELLER MOV,
            ADR METING? CLR,            \ Meetcyclus afgerond
            ADR KLAAR? SETB,
        THEN,
    THEN,
    ADR DENDER? SETB,                   \ Activeer dender onderdrukking
    RETI,
END-CODE

CODE TIJDMETING-SETUP   ( -- )
    TMOD: $01 # ORL,                    \ Timer 0 in mode 1, 16 bits timer
    TH0: -100. NIP # MOV,               \ Timer 0 start waarde
    TL0: -100. DROP # MOV,
    IP: $01 # MOV,                      \ Externe int. hoge prioriteit
    TCON: $11 # MOV,                    \ Start timer 0 & ing. flank getriggert
    IE: $83 # MOV,                      \ Zet interrupts EX0 & ET0 aan
    ADR METING? CLR,                    \ Nog geen meetcyclus begonnen
    ADR DENDER? CLR,                    \ Dender onderdrukking niet actief
    ADR KLAAR? SETB,                    \ Meetcyclus niet actief
    RET,
END-CODE

: START-METING          ( -- )
    BEGIN  10 MS  STARTKNOP 0= UNTIL    \ Knop ingedrukt
    BEGIN  10 MS  STARTKNOP UNTIL       \ Knop losgelaten
    CLEAR KLAAR?
    ;

: BEREKEN-TOERENTAL     ( -- ud )       \ Converteer periodetijd naar omw/min
    60000. 0. FROM MEETTIJD DUM/MOD 2NIP
    ;

: TOON-METING           ( -- )
    BEGIN  KLAAR? UNTIL                 \ Wacht tot meting gedaan is,
    CR ." Duur van een meting: "
    FROM MEETTIJD DU.                   \ print dan de periodetijd en
    ." millisec."
    CR ." Toerental: "
    BEREKEN-TOERENTAL DU.               \ bereken en print het toerental
    ." omw/min. "  50 MS
    ;

\ Het meetbereik is in de huidige configuratie van 1 tot 15000 toeren.
: TIJDMETING            ( -- )
    SETUP  TIJDMETING-SETUP  RS232-SETUP
    DECIMAL  CLEAR MEETTIJD  100 MS
    PAGE CR ." 'Egel toerentalmeter" CR
    BEGIN
        START-METING  TOON-METING
    AGAIN
    ;

' TEL        TF0-VEC     SET-VECTOR     \ TEL is de periodetijd timer
' NOTEER     IE0-VEC     SET-VECTOR     \ NOTEER meet de periodetijd
' TIJDMETING RESET-VEC   SET-VECTOR     \ TIJDMETING start op na een reset

\ End
