\ RC5 afstandbediening decoder en lamp aan/uit besturing, 260 bytes.

89C2051 TARGET                          \ Compileer voor AT89C2051, xt = 6 MHz

#00 CONSTANT RC-TYPE                    \ RC5 zender type
                                        \ TV = 0, CD = 20 en VCR = 5
VARIABLE RC-SYS                         \ Systeem databyte (8 bits)
VARIABLE RC-DATA                        \ Commando databyte (6 bits)
VARIABLE RC#                            \ Aantal onvangen bits

\ Geef true als er een voor dit systeem geldig RC5 commando is.
FLAG RC-KEY?  ( -- flag )

FLAG RC-CONTROLE                        \ Eerste halve bit, voor controle
FLAG RC-BIT                             \ Tweede halve bit, voor controle
$B2 BIT-SFR RC-IN                       \ Ingangs bit P3.2

CODE BEWAAR-RC-BIT  ( -- )              \ Bewaar gedecodeerde bits
    A: 16 # ANL, 
    ZER IF,                             \ Is het een systeem bit ?
        A: RC-SYS MOV,                  \ Ja, ...
        A: RLC,
        RC-SYS A: MOV,
    ELSE,                               \ Nee, het is een data bit
        A: RC-DATA MOV,
        A: RLC,
        RC-DATA A: MOV,
    THEN,
    RC# INC,                            \ Verhoog de bit teller
    RET,
END-CODE

CODE LEES-BIT       ( -- )              \ Lees en decodeer een bit
    PSW: PUSH,
    ACC: PUSH,
    TH1: -430. NIP # MOV,               \ Zet tijd tot volgende meting 860 us
    TL1: -430. DROP # MOV,              \ plus 20 us looptijd = 880 us

    C: ADR RC-IN MOV,                   \ Lees RC5 bit
    ADR RC-BIT C: MOV,                  \ Bewaar voor patroon controle
    A: RC# MOV,                         \ Haal bit teller op
    ACC: .0 BS IF,                      \ Verzamel elk even (tweede) bit ?
        ' BEWAAR-RC-BIT CALL,           \ Bewaar bit
        A: CLR,                         \ Controleer bit patroon....
        C: ADR RC-CONTROLE MOV,         \ Lees vorig RC5 bit
        A: RLC,                         \ Zet op positie 1 in accu
        C: ADR RC-BIT MOV,              \ Lees actuele RC5 bit
        A: 0 # ADDC,                    \ Tel de bits bij elkaar op
        A: 1 # EQL IF,                  \ Geldig patroon 0/1 of 1/0 opgepikt ?
            A: RC# MOV,                 \ Einde van de bitstroom
            A: $1C # EQL IF,            \ als er 14 bits opgepikt zijn ?
                RC-SYS $FF # XRL,       \ Keer system bits om
                RC-DATA $FF # XRL,      \ Keer data bits om
                RC-DATA $3F # ANL,      \ De lage 6 databits zijn geldig
                A: RC-SYS MOV,
                A: $1F # ANL,           \ Pik zender ID op
                A: RC-TYPE # EQL IF,    \ Is het een het juiste type ?
                    ADR RC-KEY? SETB,   \ Ja, commando ontvangen !
                    IE: .0 SETB,        \ EX0 interrupt weer aan
                    TCON: .6 CLR,       \ Stop timer 1
                THEN,
            THEN,
        ELSE,                           \ Ongeldig patroon, stop decoder
            IE: .0 SETB,                \ EX0 interrupt aan
            TCON: .6 CLR,               \ Stop timer 1
        THEN,
    ELSE,
        ADR RC-CONTROLE C: MOV,         \ Bewaar eerste halve bit waarde
        RC# INC,                        \ Verhoog bit teller
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

CODE RC-START      ( -- )               \ Flank gezien, start decoder
    PSW: PUSH,
    ADR RC-KEY? BC IF,                  \ Vorige data verwerkt ?
        IE: .0 CLR,                     \ Ja, EX0 interrupt uit
        TH1: -215. NIP # MOV,           \ Zet tijd tot eerste meting, 430 us 
        TL1: -215. DROP # MOV,          \ plus 24 us = 454 us
        TCON: .6 SETB,                  \ Start timer 1
        RC# 1 # MOV,                    \ Fake half ontvangen bit,
        ADR RC-CONTROLE SETB,           \ dit fake bit is hoog !!!
    THEN,
    PSW: POP,
    RETI,
END-CODE

CODE RC-SETUP      ( -- )               \ Installeer decoder hardware
    TMOD: $10 # ORL,                    \ Zet Timer 1 in mode 1
    IE: .7 SETB,                        \ Zet interrupts aan
    TCON: .6 CLR,                       \ Stop timer 1
    IE: .3 SETB,                        \ Sta timer 1 interrupts toe
    TCON: .0 SETB,                      \ EX0 actief op lage flanken
    IE: .0 SETB,                        \ EX0 interrupt aan
    ADR RC-KEY? CLR,                    \ Geen RC5 commando ontvangen
    ADR RC-IN SETB,                     \ P3.2 is de ingang
    RET,
END-CODE

\ Ontvang een RC5 databyte, x is een code uit de gedefinieerde
\ RC5-code commando set. Bekijk hiervoor de RC5 device documentatie.
: RC-KEY        ( -- x )                \ Wacht op een commando databyte
    BEGIN  RC-KEY? UNTIL                \ Wacht
    FROM RC-DATA                        \ Laat commando achter
    CLEAR RC-KEY?                       \ Sta volgend commando toe
    ;

$B5 BIT-SFR LED                         \ Een led op P3.5
$B7 BIT-SFR LAMP                        \ Het solid state relais op P3.7

: SECONDEN          ( u -- )            \ Wacht u seconden
    FOR  4 FOR  250 MS  NEXT  NEXT
    ;

: RC-LAMP           ( -- )              \ Bestuur een lamp via RC5
    SETUP  RC-SETUP  SET LAMP
    CLEAR LED  2 SECONDEN  SET LED      \ Toon opstarten
    BEGIN
        RC-KEY 12 = IF                  \ Aan/uit toets ontvangen ?
            TOGGLE LED  TOGGLE LAMP     \ Ja, schakel lamp aan/uit
            250 MS  RC-KEY? IF          \ Wacht even en gooi tweede
                RC-KEY DROP             \ code weg indien ontvangen
            THEN
        THEN
    AGAIN
    ;

\ Installeer interrupt routines

' LEES-BIT  TF1-VEC     SET-VECTOR      \ LEES-BIT via timer 1 interrupt
' RC-START  IE0-VEC     SET-VECTOR      \ RC-START geactiveerd door P3.2
' RC-LAMP   RESET-VEC   SET-VECTOR      \ RC-LAMP start op na een reset

\ Einde