\ I2C in en uitvoer met PCF8574, totale codelengte: 304/277 bytes

89C2051 TARGET                          \ Code draait op een 89C2051

\ I2C primitieven voor een 6 MHz kristal

ASSEMBLER
P3: .4  BIT-SFR SDA                     \ Serieele data lijn
P3: .5  BIT-SFR SCL                     \ Serieele klok lijn
WORK
$40 CONSTANT I/O-ID                     \ Device adres 0 van gebruikte chip

\ Algemene I2C primitieven

: SETUP-I2C      ( -- )                 \ Setup I2C I/O bits
    SET SDA  SET SCL
    ;

: START-BIT     ( -- )                  \ Genereer startbit
    SET SDA  SET SCL  CLEAR SDA  CLEAR SCL
    ;

: STOP-BIT      ( -- )                  \ Genereer stopbit
    CLEAR SDA  SET SCL  SET SDA
    ;

: ACK?          ( -- vlag )             \ Test binnenkomend ackbit, vlag is
    SET SDA  SET SCL  SDA 0=  CLEAR SCL \ true als er een ack ontvangen is
    ;

: BYTE-UIT      ( byte -- )             \ Stuur byte en test ack
    8 FOR
        DUP 128 AND  TO SDA  2*
        SET SCL  CLEAR SCL
    NEXT
    DROP  ACK? 0= IF  STOP-BIT  THEN    \ Geen ack, genereer dan een stopbit
    ;

: BYTE-IN       ( -- byte )             \ Ontvang byte
    0  8 FOR
        SET SCL  2*  SDA 1 AND  OR  
        CLEAR SCL
    NEXT
    SET SDA  SET SCL  CLEAR SCL         \ Geef no-ack
    ;

\ Uitvoer routine voor PCF8574(a) chips

: !BYTE         ( byte chip -- )
    START-BIT
    $FE AND BYTE-UIT                    \ Stuur schrijfadres naar chip en
    BYTE-UIT                            \ daarna de databyte
    STOP-BIT                            \ Klaar
    ;

\ Lees routine voor PCF8574(a) chips

: @BYTE         ( chip -- byte )
    START-BIT
    1 OR BYTE-UIT                       \ Stuur leesadres naar chip en
    BYTE-IN                             \ lees databyte van ingang
    STOP-BIT
    ;

\ De tweede I2C applicatie is een looplicht met variabele snelheid

: 10MS          ( u -- )                \ Wacht u maal 10 millisec.
    FOR  10 MS  NEXT
    ;

: INGANG        ( -- +n )               \ Lees data van PCF8574 op adres 0
    I/O-ID @BYTE INVERT  $F AND  1+ 2*  \ +n is getal tussen 2 en 32
    ;

: NAAR-LEDS     ( byte -- )             \ Stuur data naar PCF8574 op adres 0
    INVERT  $0F OR  I/O-ID !BYTE        \ De lage 4 bits zijn ingangen
    ;

: KNIPPER       ( -- )                  \ Visualiseer opstarten
    -1 NAAR-LEDS  25 10MS
    0  NAAR-LEDS  25 10MS
    ;

: LOOPLICHT     ( -- )                  \ Toon een looplicht op de leds
    SETUP  SETUP-I2C  KNIPPER
    BEGIN
        4 0 DO
            1 I 4 + LSHIFT NAAR-LEDS    \ Zet een led aan
            INGANG 10MS                 \ Wacht 20 tot 320 millisec.
        LOOP
    AGAIN
    ;

' LOOPLICHT     RESET-VEC   SET-VECTOR  \ LOOPLICHT start op na een reset
