\ Title:        High level person counter for HCC-dagen 1996
\ Created:      October 20, 1996, Willem Ouwerkerk
\ Lastchanged:  November 30, 1996, Willem Ouwerkerk
\ Uses:         ByteForth for AT89Cx051
\ Parts:        Some connectors, 1 * electronic relais, 5 * intelligent
\               7 segment digits, 1 approach sensor.
\ Example:      Use high level Forth to count persons.
\ Length:       453 bytes. opt: 399 bytes.

89C1051  TARGET                         \ Setup compiler
37 0 9 MEMORY                           \ Define memory layout
needs target.frt

DOC  About the use of pins of P1 and P3

An AT89C1051 processor is more than sufficient !!
Code written for a 6 MHz christal.

P1.0 - Led digit 1
P1.1 - Led digit 2
P1.2 - Led digit 3
P1.3 - Led digit 4
P1.4 - Led digit 5
P1.7 - Input for appoach sensor

P3.0 - Databus 0
P3.1 - Databus 1
P3.2 - Databus 2
P3.3 - Databus 3
P3.5 - Output for camera servo pulse!
P3.7 - Output for electronic relais (Active high)

Outputs are active high !!!!

ENDDOC

\ ByteForth high level servo control.

VARIABLE STAND                          \ Bevat toestand van servo interrupt
VARIABLE SERVO1-HIGH                    \ Puls lengte van servo 1
VARIABLE SERVO1-LOW 

CODE CONVERT        ( u -- d )          \ Converteer u naar een timer waarde
    A: @R0 MOV,                         \ De conversie is: NEG((u+256)*2)=d
    A: CPL,                             \ INV(u)
    R1: -2 # MOV,                       \ Hoge byte is: NEG(256)
    C: CLR,                             \ Lage byte is: -d1*2=-d2
    A: RLC,
    A: R1: XCH,                         \ Behandel hoge byte
    A: RLC,
    A: R1: XCH,                         \ Ruil hoge om met lage byte
    @R0 A: MOV,                         \ Bewaar resultaat
    A: R1: MOV,
    R0: DEC,
    @R0 A: MOV,
    RET,
END-CODE

: SERVO         ( u -- )                \ Zet nieuwe stand van de servo
    CONVERT                             \ Converteer stand
    TO SERVO1-HIGH                      \ Bewaar
    TO SERVO1-LOW  
    ;

\ Een servo routine op interrupts

CODE SERVO-INTERRUPT ( -- )             \ Aansturing voor 3 servo's
    ACC: PUSH,
    A: ADR STAND MOV,                   \ Geef interrupt toestand
    ZER IF,                             \ Is die nul ?
        P3: .5 SETB,
        TH0:  ADR SERVO1-HIGH MOV,      \ Zet pulsbreedte voor servo 1
        TL0:  ADR SERVO1-LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,
    P3: .5 CLR,
    TH0: -10000. NIP # MOV,             \ Zet pauze tijd
    TL0: -10000. DROP # MOV,
    ADR STAND 0 # MOV,                  \ Begin weer met toestand 0
    ACC: POP,
    RETI,
END-CODE

\ Zet processor hardware voor de servo interrupt.

CODE SERVO-SETUP    ( -- )              \ Zet hardware voor servo aansturing
    ADR STAND 0 # MOV,
    TMOD: $11 # MOV,
    TH0: $FF # MOV,
    TL0: $FF # MOV,
    IE: $82 # MOV,
    TCON: .4 SETB,
    RET,
END-CODE

\ Einde servo interrupt

$90 SFR SELECT                          \ Highlevel access to P1
$B0 SFR DISPLAY                         \ and P3

\ Basic routines for person counter

ATOM MS                                 \ Default is 6 MHz version
ATOM UM/MOD

: SEC                   ( u -- )        \ Wait u seconds
    FOR  10 FOR  100 MS  NEXT  NEXT
    ;

: M/MOD                 ( d1 u -- ur d2 )
    >R  0  R@ UM/MOD  R> SWAP >R  UM/MOD R> 
    ;

: DU.                   ( ud -- )       \ Print ud to 5 digit led display
    $01  5 FOR                          \ Print 5 digits, start at last
        >R  #10 M/MOD                   \ Save digit address, convert digit
        ROT  TO DISPLAY                 \ Put digit at databus
        R@ INVERT  TO SELECT            \ Put data at selected digit
        SET SELECT                      \ Deselect digit
        R>  2*                          \ Address next digit
    NEXT
    DROP  2DROP                         \ Remove digit address and ud
    ;

: LIGHT-PERSON          ( -- )          \ Put person into light
    $80 TO DISPLAY                      \ Lamp on
    150 MS                              \ Wait a moment
    020 SERVO                           \ Push camera button
    1 SEC                               \ Wait a moment
    250 SERVO                           \ Release camera button
    1 SEC                               \ Wait for an extra second
    CLEAR DISPLAY                       \ Lamp off
    ;

: PERSON-DETECTED?      ( -- flag )     \ A person detected ?
    SELECT INVERT $80 AND 0<
    ;

2VARIABLE PERSONS                       \ Persons counter

\ Main program

: COUNT-PERSONS         ( -- )          \ Count persons and put every 100
    SETUP                               \ person into light
    SERVO-SETUP
    SET SELECT                          \ Input high, no digit selected
    CLEAR DISPLAY                       \ Everything off
    CLEAR PERSONS                       \ Reset person counter
    0. DU.                              \ Clear display to all zeros
    250 SERVO                           \ Servo in begin toestand.
    BEGIN
        BEGIN  PERSON-DETECTED? UNTIL   \ Person detected ?
        INCR PERSONS                    \ Yes, increase counter
        FROM PERSONS DU.                \ and print counter contents
        FROM PERSONS 50 UM/MOD DROP 0=  \ Next fiftieth person ?
        IF  LIGHT-PERSON  THEN          \ Yes, put him/her into light
        1 SEC                           \ Wait for person to enter completely
        BEGIN PERSON-DETECTED? 0= UNTIL \ Person has left
    AGAIN
    ;

' SERVO-INTERRUPT  TF0-VEC   SET-VECTOR \ Installeer servo interrupt
' COUNT-PERSONS    RESET-VEC SET-VECTOR \ Install person counter

\ End
