\ ==============================================================
\ [TITLE]     AT80C2051 ByteForth duty cycle code (interrupt code)
\ [FILE]      PWM.FRT
\ [CATEGORY]  Application: 912 bytes, opt: 807 bytes
\ [VSN]       1.00
\ [CREATED]   December 28, 1996, Willem Ouwerkerk
\ [LASTCHNGD] May 9, 1997, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1996-1997
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   Multiple duty cycle outputs controlled by RC5 (6MHz)
\ ==============================================================
\ --------------------------------------------------------------
\ Port-1:   0 = Duty Cycle output 1
\           1 = Duty Cycle output 2 
\           2 = Duty Cycle output 3
\           3 = Duty Cycle output 4
\           4 = Duty Cycle output 5
\           5 = Duty Cycle output 6
\           6 = Duty Cycle output 7
\           7 = Duty Cycle output 8
\ --------------------------------------------------------------
\ Port-3:   0 = RC5-transmitter type 1
\           1 = RC5-command received LED
\           2 = RC5 input
\           3 = .. Possible dc output 9
\           4 = .. Possible dc output 10 
\           5 = .. Possible dc output 11
\           6 = Not present
\           7 = .. Possible dc output 12
\ --------------------------------------------------------------

  89C2051 TARGET                        \ Target compilation
  $23 8 44 MEMORY                       \ Code from $23, 8 flags, 
                                        \ 44 variables and 26 nestings
ATOM DUP        ATOM MS                 \ Import some macros
                                        \ 6 MHz version
: 10MS      ( u -- )                    \ Wait u times 10 ms
    FOR  10 MS  NEXT
    ;

$B2 BIT-SFR RC-IN                       \ Input for RC (SFH506)
$B7 BIT-SFR RC-LED                      \ Output command received
                                        \ Duty cycle:
8 =: #OUTPUTS                           \ Number of outputs
16 =: #ENTRY'S                          \ Total table length
#OUTPUTS VARIABLES PATTERN1             \ Hold individual lamp level 1
#OUTPUTS VARIABLES PATTERN2             \ Hold individual lamp level 2
#OUTPUTS VARIABLES PATTERN3             \ Hold individual lamp level 3
#OUTPUTS VARIABLES LAMPS                \ Time on ticks (0-20) 1 to #
#OUTPUTS VARIABLES WORK                 \ Time on tick counter 1 to #
VARIABLE PERIOD                         \ Reset counter
                                        \ RC5:
VARIABLE RC-SYS                         \ RC5 system databyte (8 bits)
VARIABLE RC-DATA                        \ RC5 command databyte (6 bits)
VARIABLE RC#                            \ RC5 received bit counter
                                        \ Bit-flags:
FLAG RC?                                \ RC5 command received

\ 172*1000=218*50=282900 cycles of 500.000 is  56.19 % of execution 
\ time for 8 outputs at port-1, four extra outputs at port 3 makes
\ the code more difficult or needs a rewrite of some parts.
CODE DUTY-CYCLE     ( -- )              \ Timer-0: Duty cycle control
    PSW: PUSH,                          \ Save used registers
    ACC: PUSH,
    [R0] PUSH,
    [R1] PUSH,
    [R2] PUSH,
    [R3] PUSH,
    TH0: -500. NIP # MOV,               \ Set negated 16 bit count
    TL0: -500. DROP # MOV,
    R0: ADR LAMPS # MOV,                \ Start of pulsewidth array
    R1: ADR WORK # MOV,                 \ Start of work register array
    R2: $FE # MOV,                      \ Bit mask
    R3: #OUTPUTS # MOV,                 \ Number of outputs
    DO,     
        A: @R1 MOV,                     \ Get work register #
        ZER IF,                         \ Pulse count zero ?
            A: R2: MOV,                 \ Yes, set output low
            P1: A: ANL,
        THEN,
        @R1 DEC,                        \ Decrease on count
        R1: INC,                        \ To next work register
        A: R2: MOV,                     \ Make new bit mask
        A: RL,
        R2: A: MOV,
    R3: LOOP,                           \ #OUTPUTS times
    A: ADR PERIOD MOV,                  \ PWM cycle done ?
    ZER IF,
        ADR PERIOD 20 # MOV,            \ Yes, start new cycle
        R1: ADR WORK # MOV,             \ Start of work register array
        R2: 1 # MOV,                    \ Bit mask
        R3: #OUTPUTS # MOV,             \ Number of outputs
        DO,
            A: @R0 MOV,                 \ Get pulse width PWM
            @R1 A: MOV,                 \ Copy to WORK register
            NZR IF,
                A: R2: MOV,             \ Set corresponding output high
                P1: A: ORL,
           THEN,
           R0: INC,                     \ To next pulse width variable
           R1: INC,                     \ To next work register
           A: R2: MOV,                  \ Make next bit mask
           A: RL,
           R2: A: MOV,
        R3: LOOP,                       \ #OUTPUTS times
    THEN,
    ADR PERIOD DEC,                     \ Duty cycle tick
    [R3] POP,                           \ Restore used registers
    [R2] POP,
    [R1] POP,
    [R0] POP,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

\ Initialisation for duty cycle control

CODE LAMPS-OFF      ( -- )              \ All lamps off
    A: 20 # MOV,                        \ Fill pulsewidth arrays with zero's
    R1: ADR LAMPS # MOV,
    R2: #ENTRY'S # MOV,    
    DO,     
        @R1 A: MOV,
        R1: INC,
    R2: LOOP,
    RET,
END-CODE

CODE SETUP-PWM       ( -- )              \ Maybe timer1 in auto reload mode
    ADR PERIOD 20 # MOV,                 \ Initialise duty-cycle ticks
    A: 10 # MOV,
    R1: ADR PATTERN1 # MOV,              \ Fill pulsewidth arrays with zero's
    R2: 40 # MOV,    
    DO,     
        @R1 A: MOV,
        R1: INC,
    R2: LOOP,
    TMOD: $11 # MOV,                    \ Set Timer 0 & 1 in mode 1
    TH0: -500. NIP # MOV,               \ Set negated 16 bits count
    TL0: -500. DROP # MOV,
    IE: .7 SETB,                        \ Global enable interupts
    IE: .1 SETB,                        \ Enable timer 0 interrupts
    TCON: .4 SETB,                      \ Start timer 0
    TCON: .6 CLR,                       \ Stop timer 1
    IE: .3 SETB,                        \ Enable timer 1 interrupts
    ADR RC? CLR,                        \ No RC5 data received
    TCON: .0 SETB,                      \ EX0 active on falling edges
    IE: .0 SETB,                        \ EX0 interrupt on
    IP: $09 # MOV,                      \ EX0 and TF1 high priority
    RET,
END-CODE

: LAMP-down     ( lamp# -- )            \ Lamp # more bright
    DUP FROM LAMPS 20 U< 
    IF
        DUP INCR LAMPS
    THEN
    DROP
    ;

: LAMP-up       ( lamp# -- )            \ Lamp # less bright
    DUP FROM LAMPS 
    IF
        DUP DECR LAMPS
    THEN
    DROP
    ;

: LAMPS-ON1     ( -- )                  \ Switch lamps to default state 1
    8 0 DO
        I FROM PATTERN1
        I TO LAMPS
    LOOP
    ;

: LAMPS-ON2     ( -- )                  \ Switch lamps to default state 2
    8 0 DO
        I FROM PATTERN2
        I TO LAMPS
    LOOP
    ;

: LAMPS-ON3     ( -- )                  \ Switch lamps to default state 3
    8 0 DO
        I FROM PATTERN3
        I TO LAMPS
    LOOP
    ;

: SAVE-LAMPS1   ( -- )                  \ Save current lamp state 1
    8 0 DO
        I FROM LAMPS
        I TO PATTERN1
    LOOP
    ;

: SAVE-LAMPS2   ( -- )                  \ Save current lamp state 2
    8 0 DO
        I FROM LAMPS
        I TO PATTERN2
    LOOP
    ;

: SAVE-LAMPS3   ( -- )                  \ Save current lamp state 3
    8 0 DO
        I FROM LAMPS
        I TO PATTERN3
    LOOP
    ;

\ RC5 remote control receiver

CODE START-BIT      ( -- )              \ Ext-0: Start receiving RC5 code
    ADR RC? BC IF,                      \ RC5 data processed ?
        IE: .0 CLR,                     \ Yes, EX0 interrupt off
        ADR RC# 0 # MOV,                \ Zero bits received
        TH1: -200. NIP # MOV,           \ Set negated 16 bits count
        TL1: -200. DROP # MOV,
        TCON: .6 SETB,                  \ Start timer 1
    THEN,
    RETI,
END-CODE

CODE RECEIVE        ( -- )              \ Timer-1: Read RC5 bit, etc...
    PSW: PUSH,
    ACC: PUSH,
    TH1: -880. NIP # MOV,               \ Set negated 16 bits count
    TL1: -880. DROP # MOV,
    C: ADR RC-IN MOV,                   \ Read next RC5 bit
    A: ADR RC# MOV,
    A: 8 # ANL, 
    ZER IF,                             \ Less then eight bits received ?
        A: ADR RC-SYS MOV,              \ Yes, it are system bits !
        A: RLC,
        ADR RC-SYS A: MOV,
    ELSE,                               \ No, its data !
        A: ADR RC-DATA MOV,
        A: RLC,
        ADR RC-DATA A: MOV,
    THEN,
    ADR RC# INC,                        \ Increase bit counter
    A: ADR RC# MOV,                     \ End of bitstream
    A: $E # EQL IF,                     \ 14 bits received ?
        ADR RC-SYS $FF # XRL,           \ Invert system bits
        ADR RC-DATA $FF # XRL,          \ Invert data bits
        ADR RC-DATA $3F # ANL,          \ Leave only lower 6 data bits
        A: ADR RC-SYS MOV,
        A: $1F # ANL,
        A: 05 # EQL IF,                 \ Is it a VCR remote control ?
            ADR RC? SETB,               \ Yes
        THEN,
        IE: .0 SETB,                    \ EX0 interrupt on
        TCON: .6 CLR,                   \ Stop timer 1
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

: RCKEY         ( -- command )          \ Get RC5 command databyte
    BEGIN  
        RC?                             \ Wait for rc-key
    UNTIL
    FROM RC-DATA                        \ Push command
    CLEAR RC?                           \ Make way
    ;

: HANDLE-COMMAND    ( command -- )
    BEGIN-SELECT      
        SELECT 42  0 LAMP-UP    EXIT    \ 1
        SELECT 44  0 LAMP-DOWN  EXIT
        SELECT 16  1 LAMP-UP    EXIT    \ 2
        SELECT 17  1 LAMP-DOWN  EXIT 
        SELECT 26  2 LAMP-UP    EXIT    \ 3
        SELECT 27  2 LAMP-DOWN  EXIT
        SELECT 22  3 LAMP-UP    EXIT    \ 4
        SELECT 23  3 LAMP-DOWN  EXIT
        SELECT 24  4 LAMP-UP    EXIT    \ 5
        SELECT 25  4 LAMP-DOWN  EXIT
        SELECT 18  5 LAMP-UP    EXIT    \ 6
        SELECT 19  5 LAMP-DOWN  EXIT
        SELECT 20  6 LAMP-UP    EXIT    \ 7
        SELECT 21  6 LAMP-DOWN  EXIT
        SELECT 57  7 LAMP-UP    EXIT    \ 8
        SELECT 56  7 LAMP-DOWN  EXIT
        SELECT 01  LAMPS-ON1    EXIT    \ Lamps to default on state 1
        SELECT 02  LAMPS-ON2    EXIT    \ Lamps to default on state 2
        SELECT 03  LAMPS-ON3    EXIT    \ Lamps to default on state 3
        SELECT 04  SAVE-LAMPS1  EXIT    \ Set new default lamps state 1
        SELECT 05  SAVE-LAMPS2  EXIT    \ Set new default lamps state 2
        SELECT 06  SAVE-LAMPS3  EXIT    \ Set new default lamps state 3
        SELECT 12  LAMPS-ON1    EXIT    \ Lamps to default on state 1
        SELECT 13  LAMPS-OFF    EXIT    \ All lamps off
    END-SELECT
    ;

: LAMP-CONTROL  ( -- )                  \ Control 8 PWM lamps thru RC5
    SETUP                               \ Initialise Forth,
    SETUP-PWM                           \ and PWM hardware
    LAMPS-OFF                           \ all lamps in off state
    CLEAR RC-LED                        \ Startup signal, led on
    100 10MS                            \ Wait a second
    BEGIN
        50 MS                           \ Wait
        TOGGLE RC-LED                   \ Led off
        RCKEY                           \ Wait for command
        HANDLE-COMMAND                  \ Handle function
        TOGGLE RC-LED                   \ Led on
    AGAIN
    ;

' DUTY-CYCLE      TF0-VEC   SET-VECTOR  \ Install PWM interrupt
' RECEIVE         TF1-VEC   SET-VECTOR  \ Install ontvanger interrupt
' START-BIT       IE0-VEC   SET-VECTOR  \ Install startbit interrupt
' LAMP-CONTROL    RESET-VEC SET-VECTOR  \ Install LAMP control program

\ End
