\ ==============================================================
\ [TITLE]     AT80C2051 ByteForth Puls Width Modulation
\ [FILE]      PWM8.FRT
\ [CATEGORY]  Application: 1155 bytes, opt: 983 bytes
\ [VSN]       1.32
\ [CREATED]   December 28, 1996, Willem Ouwerkerk
\ [LASTCHNGD] May 19, 1997, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1996-1997
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   Multiple PWM outputs controlled by RC5 (12MHz)
\ ==============================================================
\ Updated: RC5 receiver with pulse shape check
\          and RC5 interrupts are high priority now.
\ Updated: FASTER and SLOWER and with bounds control.
\          RC-ADJUST uses variable step size instead of 1.
\ Updated: Cycle count for PWM is 13,35 % 
\ Changed: Some names for more clarity: RC? is RC-KEY? etc.
\          Code is now for a 12 MHz xtal, was 6 MHz.
\ Changed: Cycle count for PWM now 2,71 % with early calculation,
\          and is about 5 times faster than run time calculation
\          but we need an larger array. The work is now done in
\          SET-OUTPUT which lays down a bit pattern for one output.
\ --------------------------------------------------------------
\ Port-1:   0 = PWM output 1
\           1 = PWM output 2 
\           2 = PWM output 3
\           3 = PWM output 4
\           4 = PWM output 5
\           5 = PWM output 6
\           6 = PWM output 7
\           7 = PWM output 8
\ --------------------------------------------------------------
\ Port-3:   0 = .. Possible music input or output 12
\           1 = RC5-command received LED
\           2 = RC5 input
\           3 = .. Possible zero crossing detector (sync).
\           4 = .. Possible PWM output 9 
\           5 = .. Possible PWM output 10
\           6 = Not present
\           7 = .. Possible PWM output 11
\ --------------------------------------------------------------

  89C2051 TARGET                        \ Target compilation
  $23 8 59 MEMORY                       \ Code from $23, 8 flags, 
                                        \ 47 variables and 25 nestings
needs target.frt
                                        \ Bit I/O:
$B2 BIT-SFR RC-IN                       \ Input for RC (SFH506-36)
$B7 BIT-SFR RC-LED                      \ Output command received

00 CONSTANT #RC-TYPE                    \ 0=TV 5=VCR 20=CD etc.
20 CONSTANT #CYCLE                      \ PWM cycle time 
08 CONSTANT #OUTPUTS                    \ Number of outputs
                                        \ PWM:
VARIABLE CYCLE                          \ Current output status
#CYCLE   VARIABLES STATES               \ #CYCLE (20) outputs states
#OUTPUTS VARIABLES PATTERN1             \ Private lamp levels 1
#OUTPUTS VARIABLES PATTERN2             \ Private lamp levels 2
#OUTPUTS VARIABLES PATTERN3             \ Private lamp levels 3
#OUTPUTS VARIABLES LAMPS                \ Current lamps brightness (0-#CYCLE)
                                        \ RC5:
VARIABLE RC-SYS                         \ RC5 system databyte (8 bits)
VARIABLE RC-DATA                        \ RC5 command databyte (6 bits)
VARIABLE RC#                            \ RC5 received bit counter
                                        \ Miscellaneous:
VARIABLE DELAY                          \ Hold running light step time
2VARIABLE RND                           \ Random work variable
                                        \ Bit-flags:
FLAG RC-KEY?                            \ RC5 true if command received
FLAG RC-BIT                             \ RC5 first half bit for shape control
FLAG END?                               \ Escape command flag

ADR PATTERN1 CONSTANT 'PATTERN1         \ Remember array start addresses
ADR PATTERN2 CONSTANT 'PATTERN2
ADR PATTERN3 CONSTANT 'PATTERN3

CODE MS             ( u -- )            \ Wait for u milliseconds 12 MHz
    [R1] @R0 MOV,
    R0: INC,
    DO,     R2: 166 # MOV,
            DO,     AB: MUL,
            R2: LOOP,
    R1: LOOP,
    RET,
END-CODE            

: 10MS      ( u -- )                    \ Wait u times 10 ms
    ?DUP IF  FOR  10 MS  NEXT  THEN
    ;

CODE RANDOM         ( -- u )            \ Make random number u
    A: RND MOV,
    A: RLC,                             \ Move bit 14 to bit 15 location
    A: RND LOW XRL,                     \ Xor it with bit 7 of word
    A: RLC,                             \ Save result in carry
    A: RND LOW MOV,                     \ Shift it in at bit 0 position
    A: RLC,
    RND LOW A: MOV,
    A: RND MOV,                         \ Shift bit 7 to bit 8 position
    A: RLC,
    RND A: MOV,
    A: RND LOW MOV,                     \ We use low byte as result
    R0: DEC,                            \ Push it to the stack
    @R0 A: MOV,
    RET,
END-CODE

: CHOOSE            ( u1 -- u2 )        \ Make random number u2 the
    RANDOM  UM* NIP                     \ range of u2 is 0 to u1-1
    ;

: SETUP-RANDOM      ( -- )              \ Initialise random work location
    31414. TO RND
    ;


\ (27*1000)+(2*50)=27100 cycles of 1.000.000 is 2,71 %
\ of the total execution time for 8 outputs at port-1.
CODE PWM            ( -- )              \ Timer-0: PWM control
    PSW: PUSH,                          \ Save used registers
    ACC: PUSH,
    [R1] PUSH,
    TH0: -1000. NIP # MOV,              \ Set negated 16 bit count
    TL0: -1000. DROP # MOV,
    R1: ADR CYCLE MOV,                  \ Calc. position in pulse width array
    P1: @R1 MOV,                        \ Set new output state thru pointer
    ADR CYCLE INC,                      \ To next position
    A: ADR CYCLE MOV,                   \ Get position
    A: #CYCLE STATES # EQL IF,          \ Position outside array 
        ADR CYCLE ADR STATES # MOV,     \ Yes, position at start of array
    THEN,
    [R1] POP,                           \ Restore used registers
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

\ Initialisation for PWM control

CODE SETUP-PWM       ( -- )             \ Maybe timer1 in auto reload mode
    ADR CYCLE ADR STATES # MOV,         \ PWM: To start of state array
    A: 10 # MOV,
    R1: ADR PATTERN1 # MOV,             \ Fill lamp arrays
    R2: 32 # MOV,    
    DO,     
        @R1 A: MOV,
        R1: INC,
    R2: LOOP,
    TH0: -1000. NIP # MOV,              \ Set negated 16 bits count
    TL0: -1000. DROP # MOV,
    IE: .1 SETB,                        \ Enable timer 0 interrupts
    TCON: .4 SETB,                      \ Start timer 0
\ ........  
    TCON: .6 CLR,                       \ RC5: Stop timer 1
    IE: .3 SETB,                        \ Enable timer 1 interrupts
    ADR RC-KEY? CLR,                    \ No RC5 data received
    TCON: .0 SETB,                      \ EX0 active on falling edges
    IE: .0 SETB,                        \ EX0 interrupt on
    IP: $09 # MOV,                      \ EX0 and TF1 on high priority
\ ........
    TMOD: $11 # MOV,                    \ Both: Set Timer 0 & 1 in mode 1
    IE: .7 SETB,                        \ Global enable interupts
    RET,
END-CODE

CODE SET-OUTPUT     ( lamp -- )         \ Set output states for 'lamp'.
    A: @R0 MOV,                         \ Get lamp nr. from stack
    A: INC,                             \ Correct it
    R2: A: MOV,                         \ Set shift count
    C: SETB,                            \ Start in carry
    A: CLR,                             \ Result starts at zero
    DO,
        A: RLC,                         \ Shift count times to left
    R2: LOOP,
    R2: A: MOV,                         \ Save bit mask
    A: CPL,
    R3: A: MOV,                         \ Save inverted bit mask
    A: @R0 MOV,                         \ Get lamp nr.
    R0: INC,
    A: ADR LAMPS # ADD,                 \ Add to LAMPS base addr
    R1: A: MOV,
    A: @R1 MOV,                         \ Get value of 'lamp'
    R4: A: MOV,                         \ Save lamp value
    R1: ADR STATES # MOV,               \ Set start of states address
    NZR IF,                             \ Not zero ?
        R6: A: MOV,                     \ Yes, set on state counter
        DO,
            A: @R1 MOV,                 \ Get state
            A: R2: ORL,                 \ Set bit of this state
            @R1 A: MOV,                 \ Put state back
            R1: INC,                    \ To next state
        R6: LOOP,
    THEN,
    A: #CYCLE # MOV,                    \ Total number of cycles
    C: CLR,                             \ Prepare subtract
    A: R4: SUBB,                        \ Subtract lamp on states
    NZR IF,                             \ Not zero ?
        R6: A: MOV,                     \ Yes, set off state counter
        DO,
            A: @R1 MOV,                 \ Get state
            A: R3: ANL,                 \ Clear bit of this state
            @R1 A: MOV,                 \ Put state back
            R1: INC,                    \ To next state
        R6: LOOP,
    THEN,
    RET,                                \ Done
END-CODE

: SET-OUTPUTS   ( -- )                  \ Set output states for all lamps
    8 FOR  I' 1- SET-OUTPUT  NEXT
    ;

: LAMP-down     ( lamp# -- )            \ Lamp # more bright ( UP )
    DUP FROM LAMPS #CYCLE U< 
    IF
        DUP INCR LAMPS
    THEN
    SET-OUTPUT
    ;

: LAMP-up       ( lamp# -- )            \ Lamp # less bright ( DOWN )
    DUP FROM LAMPS 
    IF
        DUP DECR LAMPS
    THEN
    SET-OUTPUT
    ;

CODE SET-LAMPS      ( level -- )        \ All lamps to level from stack
    A: @R0 MOV,
    R0: INC,
    R1: ADR LAMPS # MOV,
    R2: #OUTPUTS # MOV,    
    DO,     
        @R1 A: MOV,
        R1: INC,
    R2: LOOP,
    ' SET-OUTPUTS JMP,
END-CODE

: RESTORE-LEVEL ( addr -- )             \ Set lamps to a saved level
    8 0 DO
        DUP I +  @
        I TO LAMPS
    LOOP
    DROP  SET-OUTPUTS
    ;

: SAVE-LEVEL    ( addr -- )             \ Save current lamp levels
    8 0 DO
        I FROM LAMPS
        OVER I +  !
    LOOP
    DROP
    ;

: GET-LEVEL     ( -- l1 .. l8 )         \ Get current lamp levels
    8 0 DO
        I FROM LAMPS
    LOOP
    ;

: SET-LEVEL     ( l1 .. l8 -- )         \ Set new current lamps levels
    8 FOR
        I' 1- TO LAMPS
    NEXT
    SET-OUTPUTS
    ;

\ RC5 remote control receiver

CODE START-BIT     ( -- )               \ Detect RC start bit
    ADR RC-KEY? BC IF,                  \ Previous command used ?
        IE: .0 CLR,                     \ Yes, start decoder
        TH1: -400. NIP # MOV,
        TL1: -400. DROP # MOV,
        TCON: .6 SETB,
        ADR RC# 1 # MOV,
        ADR RC-BIT SETB,
    THEN,
    RETI,
END-CODE

CODE SAVE-RC-BIT    ( -- )              \ Save a received bit
    A: 16 # ANL, 
    ZER IF,                             \ Is it a system bit ?
        A: ADR RC-SYS MOV,              \ Yes...
        A: RLC,
        ADR RC-SYS A: MOV,
    ELSE,                               \ No it's a data bit
        A: ADR RC-DATA MOV,
        A: RLC,
        ADR RC-DATA A: MOV,
    THEN,
    RET,
END-CODE

CODE RECEIVE        ( -- )              \ Read 14 bits thru timer 0 intervals
    PSW: PUSH,
    ACC: PUSH,
    TH1: -880. NIP # MOV,               \ Set new interval
    TL1: -880. DROP # MOV,
    C: ADR RC-IN MOV,                   \ Read each half bit
    A: ADR RC# MOV,
    ACC: .0 BS IF,                      \ Collect each second bit ?
        ' SAVE-RC-BIT CALL,             \ Save received bit
        ADR RC# INC,                    \ Increase bit counter
        A: CLR,                         \ Compare bit pattern 0/1 or 1/0
        C: ADR RC-BIT MOV,
        A: RLC,
        C: ADR RC-IN MOV,
        A: 0 # ADDC,
        A: 1 # EQL IF,                  \ Valid pattern received ?
            A: ADR RC# MOV,             \ Yes
            A: 28 # EQL IF,             \ All 28 half bits received ?
                ADR RC-SYS $FF # XRL,   \ Yes, invert data
                ADR RC-DATA $FF # XRL,
                ADR RC-DATA $3F # ANL,  \ Low 6 bits are valid
                A: ADR RC-SYS MOV,
                A: $1F # ANL,
                A: #RC-TYPE # EQL IF,   \ Valid transmitter type ?
                    ADR RC-KEY? SETB,   \ Yes, correct command receieved
                THEN,
                IE: .0 SETB,            \ Stop decoder
                TCON: .6 CLR,
            THEN,
        ELSE,
            IE: .0 SETB,                \ Stop decoder
            TCON: .6 CLR,
        THEN,
    ELSE,
        ADR RC-BIT C: MOV,              \ Save first bit value
        ADR RC# INC,                    \ Increase bit counter
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

: RC-KEY        ( -- command )          \ Get RC5 command databyte
    BEGIN  RC-KEY? UNTIL                \ Wait for rc command
    FROM RC-DATA                        \ Push received command
    CLEAR RC-KEY?                       \ Key is used !
    ;

\ PWM application

: FASTER            ( n -- n )
    DUP  FROM DELAY  UMIN NEGATE        \ Keep step within range
    +TO DELAY                           \ Update delay
    ;

: SLOWER            ( n -- n )
    DUP  FROM DELAY INVERT  UMIN        \ Keep step within range
    +TO DELAY                           \ Update delay
    ;

: CHANGE-SPEED      ( -- )              \ Change effect speed
    RC-KEY
    BEGIN-SELECT
        SELECT 50  SLOWER  EXIT         \ Increase delay time
        SELECT 52  FASTER  EXIT         \ Decrease delay time
        SELECT 12  SET END?             \ Quit a function
    END-SELECT
    ;

: RC-ADJUST         ( -- )              \ Incr/decr delay time
    1                                   \ Basic step size
    BEGIN
    RC-KEY? WHILE                       \ or set END? flag !
        CHANGE-SPEED
        60 FOR                          \ Wait max 120 ms
            2 MS                        \ Wait two ms
        RC-KEY? 0= WHILE                \ RC command received
        NEXT
        ELSE  
            UNNEXT                      \ Command received
        THEN
        DUP 8 U< IF  1+  THEN           \ Max. step size is 8
    REPEAT
    DROP
    ;

: RUNNING-LIGHT     ( -- )              \ Run lights
    CLEAR END?                          \ Reset stop flag
    BEGIN                               \ Shift current lamp levels around
        GET-LEVEL  7 ROLL               \ Shuffle lamp levels
        SET-LEVEL                       \ Put changed pattern back
        FROM DELAY 10MS                 \ Stepping speed: DELAY * 10 ms
        RC-ADJUST                       \ Change stepping speed or stop
    END? UNTIL
    ;

: RANDOM-LIGHT      ( -- )              \ Random flashing lights
    CLEAR END?                          \ Reset stop flag
    BEGIN                               \ Set random lamp levels
        8 FOR  21 CHOOSE  NEXT          \ Get 8 random numbers
        SET-LEVEL                       \ Set new pattern
        RANDOM DROP                     \ Shuffle one more time
        FROM DELAY 10MS                 \ Stepping speed: DELAY * 10 ms
        RC-ADJUST                       \ Change stepping speed or stop
    END? UNTIL
    ;

: HANDLE-COMMAND    ( command -- )      \ Handle RC5 commands
    BEGIN-SELECT      
        SELECT 42  0 LAMP-UP    EXIT    \ 1
        SELECT 44  0 LAMP-DOWN  EXIT
        SELECT 16  1 LAMP-UP    EXIT    \ 2
        SELECT 17  1 LAMP-DOWN  EXIT 
        SELECT 26  2 LAMP-UP    EXIT    \ 3
        SELECT 27  2 LAMP-DOWN  EXIT
        SELECT 22  3 LAMP-UP    EXIT    \ 4
        SELECT 23  3 LAMP-DOWN  EXIT
        SELECT 24  4 LAMP-UP    EXIT    \ 5
        SELECT 25  4 LAMP-DOWN  EXIT
        SELECT 18  5 LAMP-UP    EXIT    \ 6
        SELECT 19  5 LAMP-DOWN  EXIT
        SELECT 20  6 LAMP-UP    EXIT    \ 7
        SELECT 21  6 LAMP-DOWN  EXIT
        SELECT 57  7 LAMP-UP    EXIT    \ 8
        SELECT 56  7 LAMP-DOWN  EXIT
        SELECT 01  'PATTERN1 RESTORE-LEVEL EXIT \ Lamps to level 1
        SELECT 02  'PATTERN2 RESTORE-LEVEL EXIT \ Lamps to level 2
        SELECT 03  'PATTERN3 RESTORE-LEVEL EXIT \ Lamps to level 3
        SELECT 04  'PATTERN1 SAVE-LEVEL EXIT    \ Save lamps level 1
        SELECT 05  'PATTERN2 SAVE-LEVEL EXIT    \ Save lamps level 2
        SELECT 06  'PATTERN3 SAVE-LEVEL EXIT    \ Save lamps level 3
        SELECT 12  0 SET-LAMPS EXIT              \ Lamps on
        SELECT 13  20 SET-LAMPS EXIT             \ Lamps off
        SELECT 07  RUNNING-LIGHT EXIT            \ Running light
        SELECT 08  RANDOM-LIGHT EXIT             \ Random light
    END-SELECT
    ;

: LAMP-CONTROL  ( -- )                  \ Control 8 PWM lamps thru RC5
    SETUP                               \ Initialise Forth,
    SETUP-PWM                           \ and PWM hardware
    SETUP-RANDOM                        \ Initialise random number generator
    20 SET-LAMPS                        \ all lamps in off state
    20 TO DELAY                         \ Basic delay is 200 ms
    CLEAR RC-LED                        \ Startup signal, led on
    100 10MS                            \ Wait a second
    BEGIN
        50 MS                           \ Wait
        TOGGLE RC-LED                   \ Led off
        RC-KEY                          \ Wait for command
        HANDLE-COMMAND                  \ Select function
        TOGGLE RC-LED                   \ Led on
    AGAIN
    ;

' PWM             TF0-VEC   SET-VECTOR  \ Install PWM interrupt
' RECEIVE         TF1-VEC   SET-VECTOR  \ Install receiver interrupt
' START-BIT       IE0-VEC   SET-VECTOR  \ Install start bit interrupt
' LAMP-CONTROL    RESET-VEC SET-VECTOR  \ Install LAMP control program

\ End
