\ ==============================================================
\ [TITLE]     AT89C2051 ByteForth RC5/RS232 demo code
\ [FILE]      RC-TEST.FRT
\ [CATEGORY]  Example, 881 bytes, opt: 791 bytes
\ [VSN]       1.00 
\ [CREATED]   November 18, 1995, Willem Ouwerkerk
\ [LASTCHNGD] November 30, 1996, Willem Ouwerkerk
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1996
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   RC5/RS232/LED demo for a 6 MHz cristal.
\ ==============================================================
\ The library's are imported as a whole in the code, nothing
\ is done to make the code more space efficient. Only one change
\ is made: the RC5 decoder now works on timer 0 instead of timer 1,
\ and is adjusted to work with a 6 MHz cristal freq.

89C2051 TARGET                          \ Use while testing with demo !!
needs target.frt
                                        \ TV = 0, CD = 2 and VCR = 5
VARIABLE RC-SYS                         \ RC5 system databyte (8 bits)
VARIABLE RC-DATA                        \ RC5 command databyte (6 bits)
VARIABLE RC#                            \ RC5 received bits count

\G Leave true if there is a RC5 command received.
FLAG RC-KEY?  ( -- flag )               \ RC5 command received (true)

FLAG RC-BIT                             \ RC5 first half bit, for control
$B2 BIT-SFR RC-IN                       \ RC5 input bit (port 3.2)

CODE RC-SETUP      ( -- )               \ Setup for RC5 receiving
( ) TMOD: $01 # ORL,                    \ Set Timer 0 in mode 1  Was: timer 1
( ) IE: .7 SETB,                        \ Global enable interupts
    TCON: .4 CLR,                       \ Stop timer 0  Was: timer 1
    IE: .1 SETB,                        \ Enable timer 0 interrupts
    TCON: .0 SETB,                      \ EX0 active on falling edges
    IE: .0 SETB,                        \ EX0 interrupt on
    ADR RC-KEY? CLR,                    \ No RC5 data received
    ADR RC-IN SETB,                     \ P3.2 is input
    RET,
END-CODE


\ RC5 remote control receiver (code for an 12 MHz xtal).

CODE RC-START      ( -- )               \ RC5 start receiving 
    PSW: PUSH,
    ADR RC-KEY? BC IF,                  \ RC5 data processed ?
        IE: .0 CLR,                     \ Yes, EX0 interrupt off
        TH0: -215. NIP # MOV,           \ Set negated 16 bits count  Was: timer 1
        TL0: -215. DROP # MOV,          \ Was: timer 1
        TCON: .4 SETB,                  \ Start timer 0  Was: timer 1
        RC# 1 # MOV,                    \ Half bit received
        ADR RC-BIT SETB,                \ First half bit is high!
    THEN,
    PSW: POP,
    RETI,
END-CODE  extern0

CODE SAVE-RC-BIT    ( -- )              \ Save received bit
    A: 16 # ANL, 
    ZER IF,                             \ Is it a system bit ?
        A: RC-SYS MOV,                  \ Yes
        A: RLC,
        RC-SYS A: MOV,
    ELSE,                               \ No, its a data bit
        A: RC-DATA MOV,
        A: RLC,
        RC-DATA A: MOV,
    THEN,
    RC# INC,                            \ Increase bit counter
    RET,
END-CODE

CODE READ-BIT       ( -- )              \ RC5 read bit routine
    PSW: PUSH,
    ACC: PUSH,
    TH0: -430. NIP # MOV,               \ Set negated 16 bits count  Was: timer 1
    TL0: -430. DROP # MOV,              \ Was: timer 1

    C: ADR RC-IN MOV,                   \ Read next RC5 bit
    A: RC# MOV,
    ACC: .0 BS IF,                      \ Collect every second bit ?
        ' SAVE-RC-BIT CALL,             \ Save a bit
        A: CLR,                         \ Check bit pattern !
        C: ADR RC-BIT MOV,
        A: RLC,
        C: ADR RC-IN MOV,
        A: 0 # ADDC,
        A: 1 # EQL IF,                  \ Valid pattern 0/1 of 1/0 received ?

            A: RC# MOV,                 \ End of bitstream
            A: $1C # EQL IF,            \ 14 bits received ?
                RC-SYS $FF # XRL,       \ Invert system bits
                RC-DATA $FF # XRL,      \ Invert data bits
                RC-DATA $3F # ANL,      \ Leave only lower 6 data bits
                ADR RC-KEY? SETB,       \ Yes, new data word received !
                IE: .0 SETB,            \ EX0 interrupt on
                TCON: .4 CLR,           \ Stop timer 0  Was: timer 1
            THEN,
        ELSE,
            IE: .0 SETB,                \ EX0 interrupt on
            TCON: .4 CLR,               \ Stop timer 1  Was: timer 1
        THEN,
    ELSE,
        ADR RC-BIT C: MOV,              \ Save half bit value
        RC# INC,                        \ Increase bit count
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE  timer0

\G Receive one RC5 databyte, byte is a code from the defined
\G RC5-code command set. See RC5 device documentation.
: RC-KEY        ( -- byte )             \ RC5 Get command databyte
    BEGIN  RC-KEY? UNTIL                \ Wait
    FROM RC-DATA                        \ Push command
    CLEAR RC-KEY?                       \ Make way
    ;

\ RS232 I/O with use of timer 1.

$99 CONSTANT TI:                              \ Some bit labels
$98 CONSTANT RI:
$8E CONSTANT TR1:
$AC CONSTANT ES:

\G Setup the ACIA to an RS232 baudrate of 2400 baud.
CODE SETUP-RS232    ( -- )              \ RS232 12 MHz code version
    TMOD: $20 # ORL,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: 243 # MOV,                     \ Select 1200/2400 baud
    PCON: $80 # ORL,                    \ At start at 2400 baud
    SCON: $7A # MOV,  ( $52 )           \ Serial mode 1
    ES: CLR,                            \ Disable SIO interrupt
    RET,
END-CODE

\G Output one char.
CODE RS232-EMIT     ( char -- )         \ RS232 Char to RS232
    A: @R0 MOV,
    R0: INC,
    BEGIN, TI: BS UNTIL,
    SBUF: A: MOV,
    TI: CLR,
    RET,
END-CODE

\G Output u RS232 bytes from ROM address $addr.
CODE RS232-TYPE     ( $addr u -- )      \ RS232 Rom string to RS232
    [R1] @R0 MOV,
    R0: INC,
    DPH: @R0 MOV,
    R0: INC,
    DPL: @R0 MOV,
    R0: INC,
    A: R1: MOV,
    NZR IF,
        DO,     A: CLR,
                A: @A+DPTR MOVC,
                BEGIN, TI: BS UNTIL,
                SBUF: A: MOV,
                TI: CLR,
                DPTR: INC,
        R1: LOOP,
    THEN,
    RET,
END-CODE

\G Output u RS232 bytes from RAM address addr.
CODE RS232-RTYPE    ( addr u -- )       \ RS232 Ram string to RS232
    [R2] @R0 MOV,
    R0: INC,
    [R1] @R0 MOV,
    R0: INC,
    A: R2: MOV,
    NZR IF,
        DO,     BEGIN, TI: BS UNTIL,
                SBUF: @R1 MOV,
                TI: CLR,
                R1: INC,
        R2: LOOP,
    THEN,
    RET,
END-CODE

\G Leave true is an RS232 byte is received, otherwise false.
CODE RS232-KEY?     ( -- flag )         \ RS232 Char present from RS232
    A: CLR,
    RI: BS IF,
        A: DEC,
    THEN,
    R0: DEC,
    @R0 A: MOV,
    RET,
END-CODE

\G Receive one RS232 byte char.
CODE RS232-KEY      ( -- char )         \ RS232 Read char from RS232
    BEGIN, RI: BS UNTIL,
    R0: DEC,
    @R0 SBUF: MOV,
    RI: CLR,
    RET,
END-CODE

: KEY?      RS232-KEY? ;                        ( -- flag )
: KEY       RS232-KEY ;                         ( -- char )
: EMIT      RS232-EMIT ;                        ( char -- )
: TYPE      RS232-TYPE ;                        ( $addr u -- )
: RTYPE     RS232-RTYPE ;                       ( addr u -- )
: CR        ^M RS232-EMIT  ^J RS232-EMIT ;      ( -- )
: SPACE     BL RS232-EMIT ;                     ( -- )
: SPACES    ?DUP IF  FOR  SPACE  NEXT  THEN ;   ( u -- )
: PAGE      ^L RS232-EMIT ;                     ( -- )
: BS        ^H RS232-EMIT SPACE ^H RS232-EMIT ; ( -- )

\ Number output

ATOM ROT                                \ Words wanted in dictionary
ATOM UM/MOD

VARIABLE BASE                           \ Hold number conversion base
VARIABLE HLD                            \ Work addr. in number conv. buffer
$80 CONSTANT NUM                        \ Start of number conversion buffer
                                        \ Only for AT89C2051
\ $40 CONSTANT NUM                      \ Start of number conversion buffer
                                        \ Only for AT89C1051

\ Unsigned double devide routine

\G Divide d1 by u1, leaving the single remainder u2 and the quotient d2.
: M/MOD     >R  0  R@ UM/MOD  R> SWAP >R  UM/MOD R> ;   ( d1 u1 -- u2 d2 ) \ Numbers

\ Number conversion primitives

\G Add char to the beginning of the pictured numeric output string.
: HOLD      DECR HLD  FROM HLD ! ;                      ( char -- )     \ Numbers

\G If n is negative add minus sign to the numeric output string.
: SIGN      0< IF  [CHAR] -  HOLD  THEN ;               ( n -- )        \ Numbers

\G Start pictured numeric output process.
: <#        NUM TO HLD ;                                ( -- )          \ Numbers

\G Drop d and make pictured numeric output string available as addr u.
: #>        2DROP  FROM HLD  NUM FROM HLD - ;           ( d -- addr u ) \ Numbers

\G Divide d1 by the number in BASE leaving d2. The remainder n is
\G converted to a character which is added to the begin of the
\G pictured numeric output string.
: #         FROM BASE M/MOD  ROT 9 OVER <               ( d1 -- d2 )    \ Numbers
            IF  7 +  THEN  [CHAR] 0 +  HOLD ;

\G Convert digits from d1 as described with # until the resulting
\G quotient d2 is zero.
: #S        BEGIN  # 2DUP OR 0= UNTIL ;                 ( d1 -- d2 )    \ Numbers

\ Number output words

\G Display d in free field format.
: D.        TUCK DABS <# #S  ROT SIGN #>  RTYPE SPACE ; ( d -- )        \ Numbers

\G Display n in free field format.
: .         S>D  D. ;                                   ( n -- )        \ Numbers

\G Display u in free field format.
: U.        0  D. ;                                     ( u -- )        \ Numbers

\G Set number base to 10.
: DECIMAL   #10 TO BASE ;                               ( -- )          \ Numbers

\G Set number base to 16.
: HEX       $10 TO BASE ;                               ( -- )          \ Numbers

\ End

\ Demonstration code

ATOM MS     ATOM INLINE$

$B7 BIT-SFR LED                         \ A led at P3.7
$90 SFR P1                              \ Show RC5 data

: SECONDS           ( u -- )            \ Wait half a second
    FOR  10 FOR  100 MS  NEXT  NEXT
    ;

: .HEX              ( u -- )            \ Print u in hexadecimal
    HEX  0 <# # # &$ HOLD #> RTYPE SPACE
    DECIMAL
    ;

: .DEC              ( u -- )            \ Print u decimaal
    DECIMAL  0 <# # # # &# HOLD #> RTYPE SPACE
    HEX
    ;

: FLASH-LED         ( -- )              \ Led on for a short while
    TOGGLE LED                          \ Switch led on
    100 MS                              \ Flash
    TOGGLE LED                          \ Switch led off
    ;

: RC-DEMO           ( -- )
    SETUP                               \ Basic setup
    RC-SETUP                            \ RC5 setup
    SETUP-RS232                         \ RS232 setup
    DECIMAL                             \ Afdrukken in tientallige stelsel
    PAGE ." RC5/RS232/LED demo "  CR    \ Startup sign 1
    CLEAR LED  1 SECONDS  SET LED       \ Startup sign 2
    BEGIN
        RC-KEY  DUP INVERT TO P1        \ Show RC5-command
        FLASH-LED
        ." Command: " DUP .HEX          \ Send command in hex
        .DEC                            \ and decimal
        2 SECONDS                       \ Wait
        FROM RC-SYS  DUP INVERT TO P1   \ Show system byte
        FLASH-LED
        ."   Adres: " DUP .HEX          \ Send address,
        $1F AND  .DEC  CR               \ address in decimal and cr.
    AGAIN
    ;  main

\ Install interrupt routines and demo

\ READ-BIT    TF0-VEC     SET-VECTOR      \ Install timer 0 interrupt  Was: timer 1
\ RC-START    IE0-VEC     SET-VECTOR      \ Install ext. interrupt 0
\ RC-DEMO     RESET-VEC   SET-VECTOR      \ Install startup routine

\ End
