\ Title:        RCLAMP.FRT  RC5 lamp control for HCC-dagen 1996
\ Created:      November 18, 1996, Willem Ouwerkerk
\ Lastchanged:  May 20, 1997, Willem Ouwerkerk
\ Uses:         ByteForth for AT89Cx051
\ Parts:        Some connectors, 1 * electronic relais, SFH506-36
\               resistors and Capacitators.
\ Example:      Lamps under remote control.
\ Length:       402 bytes, opt: 338 bytes.

89C2051  TARGET                         \ Setup compiler
29 2 3 MEMORY                           \ Reserve memory

DOC  About the use of pins of P1 and P3

An AT89C1051 processor should be more than sufficient !!
Timer 0 then should be used for RC5 decoding.
This code is written for a 6 MHz cristal.

P3.1            - Manual switch input
P3,2            - RC5 input
P3,3            - Led output
P3,5            - Power output
P1.4 to P1.7    - Inputs for address selection (1 of 11)

ENDDOC

#05 =: RC-TYPE                          \ RC5 Set transmitter type
                                        \ TV = 0, CD = 2 and VCR = 5

VARIABLE RC-SYS                         \ RC5 system databyte (8 bits)
VARIABLE RC-DATA                        \ RC5 command databyte (6 bits)
VARIABLE RC#                            \ RC5 received bits count

\G Leave true if there is a RC5 command received.
FLAG RC-KEY?  ( -- flag )               \ RC5 command received (true)

FLAG RC-BIT                             \ RC5 first half bit, for control
$B2 BIT-SFR RC-IN                       \ RC5 input bit (port 3.2)
$B1 BIT-SFR SWITCH?                     \ Manual switch input (port 3.1)


CODE RC-SETUP      ( -- )               \ Setup for RC5 receiving
( ) TMOD: $10 # ORL,                    \ Set Timer 1 in mode 1
( ) IE: .7 SETB,                        \ Global enable interupts
    TCON: .6 CLR,                       \ Stop timer 1
    IE: .3 SETB,                        \ Enable timer 1 interrupts
    TCON: .0 SETB,                      \ EX0 active on falling edges
    IE: .0 SETB,                        \ EX0 interrupt on
    ADR RC-KEY? CLR,                    \ No RC5 data received
\   ADR RC-IN SETB,                     \ P3.2 is input
    RET,
END-CODE


\ RC5 remote control receiver (code for an 6 MHz xtal).

CODE RC-START      ( -- )               \ RC5 start receiving 
    ADR RC-KEY? BC IF,                  \ RC5 data processed ?
        IE: .0 CLR,                     \ Yes, EX0 interrupt off
        TH1: -200. NIP # MOV,           \ Set negated 16 bits count
        TL1: -200. DROP # MOV,
        TCON: .6 SETB,                  \ Start timer 1
        RC# 1 # MOV,                    \ Half bit received
        ADR RC-BIT SETB,                \ First half bit is high!
    THEN,
    RETI,
END-CODE

CODE SAVE-RC-BIT    ( -- )              \ Save received bit
    A: 16 # ANL, 
    ZER IF,                             \ Is it a system bit ?
        A: RC-SYS MOV,                  \ Yes
        A: RLC,
        RC-SYS A: MOV,
    ELSE,                               \ No, its a data bit
        A: RC-DATA MOV,
        A: RLC,
        RC-DATA A: MOV,
    THEN,
    RC# INC,                            \ Increase bit counter
    RET,
END-CODE

CODE READ-BIT       ( -- )              \ RC5 read bit routine
    PSW: PUSH,
    ACC: PUSH,
    TH1: -440. NIP # MOV,               \ Set negated 16 bits count
    TL1: -440. DROP # MOV,
    C: ADR RC-IN MOV,                   \ Read next RC5 bit
    A: RC# MOV,
    ACC: .0 BS IF,                      \ Collect every second bit ?
        ' SAVE-RC-BIT CALL,             \ Save a bit
        A: CLR,                         \ Check bit pattern !
        C: ADR RC-BIT MOV,
        A: RLC,
        C: ADR RC-IN MOV,
        A: 0 # ADDC,
        A: 1 # EQL IF,                  \ Valid pattern 0/1 of 1/0 received ?
            A: RC# MOV,                 \ End of bitstream
            A: $1C # EQL IF,            \ 14 bits received ?
                RC-SYS $FF # XRL,       \ Invert system bits
                RC-DATA $FF # XRL,      \ Invert data bits
                RC-DATA $3F # ANL,      \ Leave only lower 6 data bits
                A: RC-SYS MOV,
                A: $1F # ANL,           \ Leave transmitter ID
                A: RC-TYPE # EQL IF,    \ Is it a .... remote control ?
                    ADR RC-KEY? SETB,   \ Yes, new data word received !
                    IE: .0 SETB,        \ EX0 interrupt on
                    TCON: .6 CLR,       \ Stop timer 1
                THEN,
            THEN,
        ELSE,
            IE: .0 SETB,                \ EX0 interrupt on
            TCON: .6 CLR,               \ Stop timer 1
        THEN,
    ELSE,
        ADR RC-BIT C: MOV,              \ Save half bit value
        RC# INC,                        \ Increase bit count
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

\G Receive one RC5 databyte, byte is a code from the defined
\G RC5-code command set. See RC5 device documentation.
: RC-KEY        ( -- byte )             \ RC5 Get command databyte
    BEGIN  RC-KEY?  SWITCH? 0=  OR      \ RC5 command or switch pushed
    UNTIL  50 MS                        \ Wait (debounce)
    SWITCH? 0= IF
        $FF                             \ Switch leaves $FF
    ELSE
        FROM RC-DATA                    \ Leave RC5 command
    THEN
    100 MS  CLEAR RC-KEY?               \ Make way 'after a second key' !
    ;

\ RC lamp control unit

$B3 BIT-SFR LED                         \ Control LED
$B5 BIT-SFR OUTPUT                      \ Lamp output
$90 SFR INPUT                           \ Lamp address input
                                        \ Maybe also RC device select

\ ATOM MS     ATOM =                      \ Import MS and =

: LAMP-ADDRESS      ( -- +n )           \ Get lamp address
    INPUT INVERT  4 RSHIFT  15 AND
    ;

: SELECT-ACTION     ( -- )              \ Select lamp function
    RC-KEY  DUP 13 =                    \ All lamps off ?
    IF  
        DROP  SET OUTPUT                \ Yes, lamp off !
    ELSE
        DUP 12 =                        \ All lamps on ?
        IF
            DROP  CLEAR OUTPUT          \ Yes, lamp on !
        ELSE
            DUP LAMP-ADDRESS  =         \ No, selected lamp address ?
            SWAP $FF =  OR              \ Or manual key press ?
            IF    
                TOGGLE OUTPUT           \ Yes, lamp on/off
            THEN
        THEN
    THEN
    ;

: RC-LAMP           ( -- )              \ Main lamp control loop
    SETUP                               \ Basic setup
    RC-SETUP                            \ RC5 setup
    CLEAR LED                           \ Led on
    250 MS                              \ for a moment
    BEGIN
        TOGGLE LED                      \ Switch led off
        SELECT-ACTION                   \ Lamp on/off
        TOGGLE LED                      \ Led on
        250 MS                          \ Wait a moment
    AGAIN
    ;

\ Install interrupt routines and demo

' READ-BIT  TF1-VEC     SET-VECTOR      \ Install timer 1 interrupt
' RC-START  IE0-VEC     SET-VECTOR      \ Install ext. interrupt 0
' RC-LAMP   RESET-VEC   SET-VECTOR      \ Install startup routine

\ End
