\ ==============================================================
\ [TITLE]     AT89C2051 ByteForth RS232 protoboard demo
\ [FILE]      RS232.FRT
\ [CATEGORY]  Test prototype board: 326 bytes, opt: 305 bytes
\ [VSN]       1.00 
\ [CREATED]   April 9, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] May 14, 1997, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1997
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   RS232 I/O on target with use of timer 1.
\ ==============================================================

89C2051  TARGET                         \ Setup compiler
2 0 0 MEMORY                            \ Define memory layout

$99 =: SBUF:                            \ Labels for 80Cxx
$99 =: TI:
$98 =: RI:
$87 =: PCON:
$89 =: TMOD:
$8D =: TH1:
$8E =: TR1:
$98 =: SCON:
$AC =: ES:
$D8 =: ADCON:

6 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 6 MHz christal

CODE RS232-SETUP
    TMOD: $20 # MOV,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: 243 # MOV,                     \ Select 1200/2400 baud
    PCON: $80 # ORL,                    \ At start 2400 baud
\   PCON: $7F # ANL,                    \ At start 1200 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    ES: CLR,                            \ Disable SIO interrupt
    RET,
END-CODE

[THEN]

12 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 12 MHz christal, for
\ a 6 MHz christal the baudrates are 1200 or 2400 baud.

CODE RS232-SETUP
    TMOD: $20 # MOV,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: 243 # MOV,                     \ Select 2400/4800 baud
\   PCON: $80 # ORL,                    \ At start 4800 baud
    PCON: $7F # ANL,                    \ At start 2400 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    ES: CLR,                            \ Disable SIO interrupt
    RET,
END-CODE

[THEN]

11 CRYSTAL? [IF]

\ All baudrates mentioned are for a 11.059 MHz christal.

CODE RS232-SETUP
    TMOD: $20 # MOV,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: $FD # MOV,                     \ Select 9600/19K2 baud
\   PCON: $80 # ORL,                    \ At start 19K2 baud
    PCON: $7F # ANL,                    \ At start 9600 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    ES: CLR,                            \ Disable SIO interrupt
    RET,
END-CODE

[THEN]

CODE RS232-EMIT     ( char -- )
    A: @R0 MOV,
    R0: INC,
    BEGIN, TI: BS UNTIL,
    SBUF: A: MOV,
    TI: CLR,
    RET,
END-CODE

CODE RS232-TYPE     ( $addr u -- )
    [R1] @R0 MOV,
    R0: INC,
    DPH: @R0 MOV,
    R0: INC,
    DPL: @R0 MOV,
    R0: INC,
    A: R1: MOV,
    NZR IF,
        DO,     A: CLR,
                A: @A+DPTR MOVC,
                BEGIN, TI: BS UNTIL,
                SBUF: A: MOV,
                TI: CLR,
                DPTR: INC,
        R1: LOOP,
    THEN,
    RET,
END-CODE

CODE RS232-RTYPE    ( addr u -- )
    [R2] @R0 MOV,
    R0: INC,
    [R1] @R0 MOV,
    R0: INC,
    A: R2: MOV,
    NZR IF,
        DO,     BEGIN, TI: BS UNTIL,
                SBUF: @R1 MOV,
                TI: CLR,
                R1: INC,
        R2: LOOP,
    THEN,
    RET,
END-CODE

RI: BIT-SFR RS232-KEY?     ( -- flag )

CODE RS232-KEY      ( -- char )
    BEGIN, RI: BS UNTIL,
    R0: DEC,
    @R0 SBUF: MOV,
    RI: CLR,
    RET,
END-CODE

: KEY?      RS232-KEY? ;                        ( -- flag )
: KEY       RS232-KEY ;                         ( -- char )
: EMIT      RS232-EMIT ;                        ( char -- )
: TYPE      RS232-TYPE ;                        ( $addr u -- )
: RTYPE     RS232-RTYPE ;                       ( addr u -- )
: CR        ^M RS232-EMIT  ^J RS232-EMIT ;      ( -- )
: SPACE     BL RS232-EMIT ;                     ( -- )
: SPACES    ?DUP IF  FOR  SPACE  NEXT  THEN ;   ( u -- )
: PAGE      ^L RS232-EMIT ;                     ( -- )
: BS        ^H RS232-EMIT SPACE ^H RS232-EMIT ; ( -- )


\ A small RS232 text demo

ATOM INLINE$

: HANDLE-INPUT      ( -- )
    BEGIN
        CR ." > " KEY
        DUP &A = IF  ." Aap "   THEN
        DUP &B = IF  ." Boef "  THEN
        DUP &C = IF  ." Chip "  THEN
    &Q = UNTIL
    ;

: TEXT-DEMO         ( -- )
    SETUP
    RS232-SETUP
    BEGIN
        PAGE ." RS232 test for protoboard VSN 1.00"
        HANDLE-INPUT
    AGAIN
    ;

' TEXT-DEMO     RESET-VEC  SET-VECTOR   \ Startup demo

\ End
