\ ==============================================================
\ [TITLE]     AT89Cx051 ByteForth high level servo control.
\ [FILE]      SERVO.FRT
\ [CATEGORY]  Demo, length: 752 bytes, opt: 693 bytes
\ [VSN]       1.12
\ [CREATED]   May 2, 1996, Willem Ouwerkerk
\ [LASTCHNGD] December 2, 1996, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1996
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   Demo for ByteForth cross compiler (12MHz)
\ ==============================================================

89C2051  TARGET                         \ Zet 'target' compilatie mode

$89             SFR TMOD                \ Declareer 8052 special function registers
$8C             SFR TH0 
$8A             SFR TL0
$A9             BIT-SFR ET0             \ 8052 bit i/o
$8C             BIT-SFR TR0    
$AF             BIT-SFR EAL 
$B9             BIT-SFR PT0

#00 =: RC-TYPE                          \ Selecteer gebruikte zender type
                                        \ TV = 0, CD = 2 en VCR = 5

CODE MS             ( u -- )            \ Wacht u milliseconden
    [R1] @R0 MOV,
    R0: INC,
    DO,     R2: 166 # MOV,
            DO,     AB: MUL,
            R2: LOOP,
    R1: LOOP,
    RET,
END-CODE            

: 10MS              ( u -- )            \ Wacht u * 10 milliseconden
    FOR  10 MS  NEXT
    ;

\ Einde algemene code


\ AT89C2051 ByteForth RC5 afstandbediening decoder.

VARIABLE RC-SYS                         \ Bevat 8 bits systeem byte
VARIABLE RC-DATA                        \ Bevat 6 bits data byte
VARIABLE RC#                            \ Aantal ontvangen bits
FLAG RC-KEY?                            \ Bevat true na goede data ontvangst
FLAG RC-BIT                             \ Bevat eerste bit helft ter controle
$B2 BIT-SFR RC-IN                       \ Invoer bit is poort bit P3.2

CODE START-BIT     ( -- )               \ Detecteer start bit
    ADR RC-KEY? BC IF,                  \ Vorige data afgehandeld ?
        IE: .0 CLR,                     \ Ja, start decoder
        TH1: -400. NIP # MOV,
        TL1: -400. DROP # MOV,
        TCON: .6 SETB,
        adr RC# 1 # MOV,
        ADR RC-BIT SETB,
    THEN,
    RETI,
END-CODE

CODE SAVE-RC-BIT    ( -- )              \ Bewaar het ontvangen bit
    A: 16 # ANL, 
    ZER IF,                             \ Is het een systeem bit ?
        A: adr RC-SYS MOV,
        A: RLC,
        adr RC-SYS A: MOV,
    ELSE,                               \ Nee het is een data bit
        A: adr RC-DATA MOV,
        A: RLC,
        adr RC-DATA A: MOV,
    THEN,
    adr RC# INC,                        \ Verhoog bit teller
    RET,
END-CODE

CODE RECEIVE        ( -- )              \ Lees 14 bits met een vaste tijdinterval
    PSW: PUSH,
    ACC: PUSH,
    TH1: -880. NIP # MOV,               \ Zet nieuwe interval
    TL1: -880. DROP # MOV,
    C: ADR RC-IN MOV,                   \ Lees een bit
    A: adr RC# MOV,
    ACC: .0 BS IF,                      \ Verzamel alleen elk tweede bit
        ' SAVE-RC-BIT CALL,             \ Bewaar ontvangen bit
        A: CLR,
        C: ADR RC-BIT MOV,
        A: RLC,
        C: ADR RC-IN MOV,
        A: 0 # ADDC,
        A: 1 # EQL IF,                  \ Geldig patroon 0/1 of 1/0 ontvangen ?
            A: adr RC# MOV,            
            A: #28 # EQL IF,            \ Alle 28 bits ontvangen ?
                adr RC-SYS $FF # XRL,   \ Ja, inverteer data
                adr RC-DATA $FF # XRL,
                adr RC-DATA $3F # ANL,  \ Lage 6 bits zijn geldig
                A: adr RC-SYS MOV,
                A: $1F # ANL,
                A: RC-TYPE # EQL IF,    \ Geldig zender type ?
                    ADR RC-KEY? SETB,   \ Ja, ontvangst ok.
                THEN,
                IE: .0 SETB,            \ Stop decoder
                TCON: .6 CLR,
            THEN,
        ELSE,
            IE: .0 SETB,                \ Stop decoder
            TCON: .6 CLR,
        THEN,
    ELSE,
        ADR RC-BIT C: MOV,              \ Bewaar eerste bit waarde
        adr RC# INC,                    \ Verhoog bit teller
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

: RC-KEY        ( -- byte )             \ Geef RC toets aanslag
    BEGIN  RC-KEY? UNTIL
    FROM RC-DATA
    CLEAR RC-KEY?
    ;

\ Installeer RC5 ontvanger interrupt routines

RECEIVE     TF1-VEC     SET-VECTOR      \ Installeer ontvanger interrupt
START-BIT   IE0-VEC     SET-VECTOR      \ Installeer startbit interrupt

\ Einde RC5 decoder


\ AT89C2051 ByteForth servo besturing.

#-20000.    2CONSTANT REITERATE         \ Herhalingstijd is 20 ms

VARIABLE STAND                          \ Bevat toestand van servo interrupt
2VARIABLE SERVO-1                       \ Puls lengte van servo 1
2VARIABLE SERVO-2                       \ Puls lengte van servo 2
2VARIABLE SERVO-3                       \ Puls lengte van servo 3
2VARIABLE PAUSE                         \ De pauze tijd is:


DOC umentatie

P1.0        = Servo uitgang 1
P1.1        = Servo uitgang 2
P1.2        = Servo uitgang 3

ENDDOC

: CALCULATE     ( -- )                  \ Bereken en zet de pauze tijd
    REITERATE
    FROM SERVO-1  D-
    FROM SERVO-2  D-
    FROM SERVO-3  D-
    TO PAUSE
    ; 

CODE CONVERT        ( u -- d )          \ Converteer u naar een timer waarde
    A: @R0 MOV,                         \ De conversie is: NEG((u+256)*4)=d
    A: CPL,                             \ INV(u)
    R1: -2 # MOV,                       \ Hoge byte is: NEG(256)
    C: CLR,                             \ Lage byte is: -d1*2=-d2
    A: RLC,
    A: R1: XCH,                         \ Behandel hoge byte
    A: RLC,
    C: CLR,
    A: R1: XCH,                         \ Lage byte is: -d2*2=d
    A: RLC,
    A: R1: XCH,                         \ Behandel hoge byte
    A: RLC,
    A: R1: XCH,
    @R0 A: MOV,                         \ Bewaar resultaat
    A: R1: MOV,
    R0: DEC,
    @R0 A: MOV,
    RET,
END-CODE

: SERVO1        ( u -- )                \ Zet nieuwe stand van servo 1
    CONVERT                             \ Converteer stand
    TO SERVO-1                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

: SERVO2        ( u -- )                \ Zet nieuwe stand van servo 2
    CONVERT                             \ Converteer stand
    TO SERVO-2                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

: SERVO3        ( u -- )                \ Zet nieuwe stand van servo 3
    CONVERT                             \ Converteer stand
    TO SERVO-3                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

\ Een servo routine op interrupts

CODE SERVO-INTERRUPT ( -- )             \ Aansturing voor 3 servo's
    ACC: PUSH,
    A: ADR STAND MOV,                   \ Geef interrupt toestand
    ZER IF,                             \ Is die nul ?
        P1: .0 SETB,
        ADR TH0  ADR SERVO-1 MOV,       \ Zet pulsbreedte voor servo 1
        ADR TL0  ADR SERVO-1 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,
    A: DEC,                             \ Is de toestand 1 ?
    ZER IF,
        P1: .0 CLR,
        P1: .1 SETB,
        ADR TH0  ADR SERVO-2 MOV,       \ Zet pulsbreedte voor servo 2
        ADR TL0  ADR SERVO-2 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,
    A: DEC,                             \ Is toestand 2 ?
    ZER IF,
        P1: .1 CLR,
        P1: .2 SETB,
        ADR TH0  ADR SERVO-3 MOV,       \ Zet pulsbreedte voor servo 3
        ADR TL0  ADR SERVO-3 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,                               \ De toestand is 3
    P1: .2 CLR,
    ADR TH0  ADR PAUSE MOV,             \ Zet pauze tijd
    ADR TL0  ADR PAUSE LOW MOV,
    ADR STAND 0 # MOV,                  \ Begin weer met toestand 0
    ACC: POP,
    RETI,
END-CODE

' SERVO-INTERRUPT TF0-VEC   SET-VECTOR  \ Installeer servo interrupt

\ Zet processor hardware voor RC5 decoder en servo interrupt.
\ Eerste voorbeeld RC-SETUP in machinecode, tweede in high level Forth.

CODE RC-SETUP      ( -- )               \ Zet hardware voor RC5 decoder
    TCON: .6 CLR,
    IE: .3 SETB,
    TCON: .0 SETB,
    IE: .0 SETB,
    ADR RC-IN SETB,
    ADR RC-KEY? CLR,
    RET,
END-CODE

: SERVO-SETUP       ( -- )              \ Zet hardware voor servo aansturing
    CLEAR STAND
    $11 TO TMOD
    SET TH0
    SET TL0
    SET EAL
    SET ET0
    SET TR0
    SET PT0
    ;

\ Einde servo interrupt


\ Servo besturing met RC5 afstandbediening.

VARIABLE MOTOR1                         \ Drie motor posities
VARIABLE MOTOR2
VARIABLE MOTOR3

: (MOVE-SERVO)      ( n addr -- )       \ Verplaats servo
    TUCK @  OVER 0<                     \ 'n' negatief ?
    IF
        SWAP NEGATE  UMIN NEGATE        \ Lage grens is nul
    ELSE
        INVERT UMIN                     \ High grens is 255
    THEN
    SWAP +!                             \ Wijzig inhoud van 'addr'
    FROM MOTOR1  SERVO1                 \ en van alle servo's
    FROM MOTOR2  SERVO2
    FROM MOTOR3  SERVO3
    ;

: RC-WAIT?      ( -- flag )             \ Wacht max. 250 MS op een rc-toets
    250 FOR
        1 MS
        RC-KEY? ?DUP 
        IF  UNNEXT EXIT  THEN           \ Toets ontvangen, klaar
    NEXT
    FALSE                               \ Geen toets ontvangen
    ;

: MOVE-SERVO    ( n addr -- )           \ Wijzig servo positie
    BEGIN
        2DUP (MOVE-SERVO)  RC-WAIT?     \ Nog een toets ontvangen ?
    WHILE
        SWAP                            \ Pas stapgrootte aan
        DUP 0< 
        IF  
            DUP -32 > IF  1-  THEN      \ Niet kleiner dan -32
        ELSE  
            DUP 32 < IF  1+  THEN       \ Niet groter dan 32
        THEN
        SWAP
        CLEAR RC-KEY?
    REPEAT
    2DROP
    ;

: MOVE-A-SERVO  ( command -- )          \ Verander de stand van een servo
    BEGIN-SELECT      
        SELECT 16   2 MOTOR1 MOVE-SERVO  EXIT 
        SELECT 17  -2 MOTOR1 MOVE-SERVO  EXIT
        SELECT 26   2 MOTOR2 MOVE-SERVO  EXIT
        SELECT 27  -2 MOTOR2 MOVE-SERVO  EXIT
        SELECT 22   2 MOTOR3 MOVE-SERVO  EXIT
        SELECT 23  -2 MOTOR3 MOVE-SERVO  EXIT
    END-SELECT
    ;

$B0 BIT-SFR LED                         \ Toon toets aanslag op ledje

: SERVO-CONTROL ( -- )                  \ Bestuur servo's via afstand bediening
    SETUP                               \ Initialiseer
    SERVO-SETUP
    RC-SETUP
    CLEAR MOTOR1                        \ Start servo's op positie nul
    CLEAR MOTOR2  
    CLEAR MOTOR3

    0 MOTOR1 MOVE-SERVO                 \ Initialiseer servo systeem
    BEGIN
        TOGGLE LED                      \ Zet led aan of uit
        RC-KEY                          \ Wacht op commando
        MOVE-A-SERVO                    \ Verplaats een servo
    AGAIN
    ;

' SERVO-CONTROL   RESET-VEC SET-VECTOR  \ Installeer voorbeeld programma

\ Einde
