\ ==============================================================
\ [TITLE]     AT89Cx051 ByteForth high level servo control.
\ [FILE]      SERVO6.FRT
\ [CATEGORY]  Application, length: 958 bytes, opt: 892 bytes
\ [VSN]       1.13
\ [CREATED]   May 2, 1996, Willem Ouwerkerk
\ [LASTCHNGD] December 9, 1996, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1996
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   Demo for ByteForth cross compiler (6MHz)
\ ==============================================================

89C2051  TARGET                         \ Zet 'target' compilatie mode
29 2 26 MEMORY                          \ En geheugen indeling

$89             SFR TMOD                \ Declareer 8052 special function registers
$8C             SFR TH0 
$8A             SFR TL0
$A9             BIT-SFR ET0             \ 8052 bit i/o
$8C             BIT-SFR TR0    
$AF             BIT-SFR EAL 
$B9             BIT-SFR PT0

#00 =: RC-TYPE                          \ Selecteer gebruikte zender type
                                        \ TV = 0, CD = 2 en VCR = 5
ATOM MS                                 \ Import MS macro

: 10MS              ( u -- )            \ Wacht u * 10 milliseconden
    FOR  10 MS  NEXT
    ;

: D-            ( d1 d2 -- d3 )         \ Trek d2 van d1 af, resultaat is d3
    DNEGATE  D+
    ;

\ Einde algemene code

\ AT89C2051 ByteForth RC5 afstandbediening decoder.

VARIABLE RC-SYS                         \ Bevat 8 bits systeem byte
VARIABLE RC-DATA                        \ Bevat 6 bits data byte
VARIABLE RC#                            \ Aantal ontvangen bits
FLAG RC-KEY?                            \ Bevat true na goede data ontvangst
FLAG RC-BIT                             \ Bevat eerste bit helft ter controle
$B2 BIT-SFR RC-IN                       \ Invoer bit is poort bit P3.2

CODE START-BIT     ( -- )               \ Detecteer start bit
    ADR RC-KEY? BC IF,                  \ Vorige data afgehandeld ?
        IE: .0 CLR,                     \ Ja, start decoder
        TH1: -200. NIP # MOV,
        TL1: -200. DROP # MOV,
        TCON: .6 SETB,
        adr RC# 1 # MOV,
        ADR RC-BIT SETB,
    THEN,
    RETI,
END-CODE

CODE SAVE-RC-BIT    ( -- )              \ Bewaar het ontvangen bit
    A: 16 # ANL, 
    ZER IF,                             \ Is het een systeem bit ?
        A: adr RC-SYS MOV,
        A: RLC,
        adr RC-SYS A: MOV,
    ELSE,                               \ Nee het is een data bit
        A: adr RC-DATA MOV,
        A: RLC,
        adr RC-DATA A: MOV,
    THEN,
    adr RC# INC,                        \ Verhoog bit teller
    RET,
END-CODE

CODE RECEIVE        ( -- )              \ Lees 14 bits met een vaste tijdinterval
    PSW: PUSH,
    ACC: PUSH,
    TH1: -440. NIP # MOV,               \ Zet nieuwe interval
    TL1: -440. DROP # MOV,
    C: ADR RC-IN MOV,                   \ Lees een bit
    A: adr RC# MOV,
    ACC: .0 BS IF,                      \ Verzamel alleen elk tweede bit
        ' SAVE-RC-BIT CALL,             \ Bewaar ontvangen bit
        A: CLR,
        C: ADR RC-BIT MOV,
        A: RLC,
        C: ADR RC-IN MOV,
        A: 0 # ADDC,
        A: 1 # EQL IF,                  \ Geldig patroon 0/1 of 1/0 ontvangen ?
            A: adr RC# MOV,            
            A: #28 # EQL IF,            \ Alle 28 bits ontvangen ?
                adr RC-SYS $FF # XRL,   \ Ja, inverteer data
                adr RC-DATA $FF # XRL,
                adr RC-DATA $3F # ANL,  \ Lage 6 bits zijn geldig
                A: adr RC-SYS MOV,
                A: $1F # ANL,
                A: RC-TYPE # EQL IF,    \ Geldig zender type ?
                    ADR RC-KEY? SETB,   \ Ja, ontvangst ok.
                THEN,
                IE: .0 SETB,            \ Stop decoder
                TCON: .6 CLR,
            THEN,
        ELSE,
            IE: .0 SETB,                \ Stop decoder
            TCON: .6 CLR,
        THEN,
    ELSE,
        ADR RC-BIT C: MOV,              \ Bewaar eerste bit waarde
        adr RC# INC,                    \ Verhoog bit teller
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

: RC-KEY        ( -- byte )             \ Geef RC toets aanslag
    BEGIN  RC-KEY? UNTIL
    FROM RC-DATA
    CLEAR RC-KEY?
    ;

\ Installeer RC5 ontvanger interrupt routines

RECEIVE     TF1-VEC     SET-VECTOR      \ Installeer ontvanger interrupt
START-BIT   IE0-VEC     SET-VECTOR      \ Installeer startbit interrupt

\ Einde RC5 decoder

\ AT89C2051 ByteForth servo besturing.

#-10000.    2CONSTANT REITERATE         \ Herhalingstijd is 20 ms

VARIABLE STAND                          \ Bevat toestand van servo interrupt
2VARIABLE SERVO-1                       \ Puls lengte van servo 1
2VARIABLE SERVO-2                       \ Puls lengte van servo 2
2VARIABLE SERVO-3                       \ Puls lengte van servo 3
2VARIABLE SERVO-4                       \ Puls lengte van servo 4
2VARIABLE SERVO-5                       \ Puls lengte van servo 5
2VARIABLE SERVO-6                       \ Puls lengte van servo 6
2VARIABLE PAUSE                         \ De pauze tijd is:

DOC umentatie

P1.2        = Servo uitgang 1
P1.3        = Servo uitgang 2
P1.4        = Servo uitgang 3
P1.5        = Servo uitgang 4
P1.6        = Servo uitgang 5
P1.7        = Servo uitgang 6

P3.2        = RC5 puls ingang
P3.7        = RC5 commando ontvangst signalering met led

ENDDOC

: CALCULATE     ( -- )                  \ Bereken en zet de pauze tijd
    REITERATE
    FROM SERVO-1  D-
    FROM SERVO-2  D-
    FROM SERVO-3  D-
    FROM SERVO-4  D-
    FROM SERVO-5  D-
    FROM SERVO-6  D-
    TO PAUSE
    ; 

CODE CONVERT        ( u -- d )          \ Converteer u naar een timer waarde
    A: @R0 MOV,                         \ De conversie is: NEG((u+256)*2)=d
    A: CPL,                             \ INV(u)
    C: CLR,                             \ Lage byte is: -d1*2=-d2
    A: RLC,
    @R0 A: MOV,                         \ Bewaar lage byte
    A: -2 # MOV,                        \ Hoge byte is: NEG(256)
    A: RLC,
    R0: DEC,
    @R0 A: MOV,                         \ Bewaar hoge byte
    RET,
END-CODE

: SERVO1        ( u -- )                \ Zet nieuwe stand van servo 1
    CONVERT                             \ Converteer stand
    TO SERVO-1                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

: SERVO2        ( u -- )                \ Zet nieuwe stand van servo 2
    CONVERT                             \ Converteer stand
    TO SERVO-2                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

: SERVO3        ( u -- )                \ Zet nieuwe stand van servo 3
    CONVERT                             \ Converteer stand
    TO SERVO-3                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

: SERVO4        ( u -- )                \ Zet nieuwe stand van servo 4
    CONVERT                             \ Converteer stand
    TO SERVO-4                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

: SERVO5        ( u -- )                \ Zet nieuwe stand van servo 5
    CONVERT                             \ Converteer stand
    TO SERVO-5                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

: SERVO6        ( u -- )                \ Zet nieuwe stand van servo 6
    CONVERT                             \ Converteer stand
    TO SERVO-6                          \ Bewaar
    CALCULATE                           \ Bereken pauze tijd
    ;

\ Een servo routine op interrupts

CODE SERVO-INTERRUPT ( -- )             \ Aansturing voor 3 servo's
    ACC: PUSH,
    A: ADR STAND MOV,                   \ Geef interrupt toestand
    ZER IF,                             \ Is die nul ?
        P1: .2 SETB,
        ADR TH0  ADR SERVO-1 MOV,       \ Zet pulsbreedte voor servo 1
        ADR TL0  ADR SERVO-1 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,
    A: DEC,                             \ Is de toestand 1 ?
    ZER IF,
        P1: .2 CLR,
        P1: .3 SETB,
        ADR TH0  ADR SERVO-2 MOV,       \ Zet pulsbreedte voor servo 2
        ADR TL0  ADR SERVO-2 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,
    A: DEC,                             \ Is toestand 2 ?
    ZER IF,
        P1: .3 CLR,
        P1: .4 SETB,
        ADR TH0  ADR SERVO-3 MOV,       \ Zet pulsbreedte voor servo 3
        ADR TL0  ADR SERVO-3 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,
    A: DEC,                             \ Is toestand 3 ?
    ZER IF,
        P1: .4 CLR,
        P1: .5 SETB,
        ADR TH0  ADR SERVO-4 MOV,       \ Zet pulsbreedte voor servo 4
        ADR TL0  ADR SERVO-4 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,
    A: DEC,                             \ Is toestand 4 ?
    ZER IF,
        P1: .5 CLR,
        P1: .6 SETB,
        ADR TH0  ADR SERVO-5 MOV,       \ Zet pulsbreedte voor servo 5
        ADR TL0  ADR SERVO-5 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,
    A: DEC,                             \ Is toestand 5 ?
    ZER IF,
        P1: .6 CLR,
        P1: .7 SETB,
        ADR TH0  ADR SERVO-6 MOV,       \ Zet pulsbreedte voor servo 6
        ADR TL0  ADR SERVO-6 LOW MOV,
        ADR STAND INC,                  \ Naar volgende toestand
        ACC: POP,
        RETI,
    THEN,                               \ De toestand is 6
    P1: .7 CLR,
    ADR TH0  ADR PAUSE MOV,             \ Zet pauze tijd
    ADR TL0  ADR PAUSE LOW MOV,
    ADR STAND 0 # MOV,                  \ Begin weer met toestand 0
    ACC: POP,
    RETI,
END-CODE

' SERVO-INTERRUPT TF0-VEC   SET-VECTOR  \ Installeer servo interrupt

\ Zet processor hardware voor RC5 decoder en servo interrupt.
\ Eerste voorbeeld RC-SETUP in machinecode, tweede in high level Forth.

CODE RC-SETUP      ( -- )               \ Zet hardware voor RC5 decoder
    TCON: .6 CLR,
    IE: .3 SETB,
    TCON: .0 SETB,
    IE: .0 SETB,
    ADR RC-IN SETB,
    ADR RC-KEY? CLR,
    RET,
END-CODE

: SERVO-SETUP       ( -- )              \ Zet hardware voor servo aansturing
    CLEAR STAND
    $11 TO TMOD                         \ Zet timer 0 & 1 in mode 1
    SET TH0
    SET TL0
    SET EAL
    SET ET0
    SET TR0
    SET PT0
    ;

\ Einde servo interrupt

\ Servo besturing met RC5 afstandbediening.

VARIABLE MOTOR1                         \ Zes motor posities
VARIABLE MOTOR2
VARIABLE MOTOR3
VARIABLE MOTOR4
VARIABLE MOTOR5
VARIABLE MOTOR6

: UPDATE-SERVOS     ( -- )              \ Stel alle servo's bij
    FROM MOTOR1  SERVO1
    FROM MOTOR2  SERVO2
    FROM MOTOR3  SERVO3
    FROM MOTOR4  SERVO4
    FROM MOTOR5  SERVO5
    FROM MOTOR6  SERVO6
    ;

: (MOVE-SERVO)      ( n addr -- )       \ Verplaats servo
    TUCK @  OVER 0<                     \ 'n' negatief ?
    IF
        SWAP NEGATE  UMIN NEGATE        \ Lage grens is nul
    ELSE
        INVERT UMIN                     \ Hoge grens is 255
    THEN
    SWAP +!                             \ Wijzig inhoud van 'addr'
    UPDATE-SERVOS
    ;

: RC-WAIT?      ( -- flag )             \ Wacht max. 250 MS op een rc-toets
    250 FOR
        1 MS
        RC-KEY? ?DUP 
        IF  UNNEXT EXIT  THEN           \ Toets ontvangen, klaar
    NEXT
    FALSE                               \ Geen toets ontvangen
    ;

: MOVE-SERVO    ( n addr -- )           \ Wijzig servo positie
    BEGIN
        2DUP (MOVE-SERVO)  RC-WAIT?     \ Nog een toets ontvangen ?
    WHILE
        SWAP                            \ Pas stapgrootte aan
        DUP 0< 
        IF  
            DUP -32 > IF  1-  THEN      \ Niet kleiner dan -32
        ELSE  
            DUP 32 < IF  1+  THEN       \ Niet groter dan 32
        THEN
        SWAP
        CLEAR RC-KEY?
    REPEAT
    2DROP
    ;

: NULL-SERVOS   ( -- )                  \ Start servo's op positie nul
    CLEAR MOTOR1
    CLEAR MOTOR2  
    CLEAR MOTOR3
    CLEAR MOTOR4
    CLEAR MOTOR5
    CLEAR MOTOR6
    UPDATE-SERVOS                       \ Initialiseer servo systeem
    ;

: MOVE-A-SERVO  ( command -- )          \ Verander de stand van een servo
    BEGIN-SELECT      
        SELECT 16   2 MOTOR1 MOVE-SERVO EXIT 
        SELECT 17  -2 MOTOR1 MOVE-SERVO EXIT
        SELECT 26   2 MOTOR2 MOVE-SERVO EXIT
        SELECT 27  -2 MOTOR2 MOVE-SERVO EXIT
        SELECT 22   2 MOTOR3 MOVE-SERVO EXIT
        SELECT 23  -2 MOTOR3 MOVE-SERVO EXIT
        SELECT 24   2 MOTOR4 MOVE-SERVO EXIT
        SELECT 25  -2 MOTOR4 MOVE-SERVO EXIT
        SELECT 18   2 MOTOR5 MOVE-SERVO EXIT
        SELECT 19  -2 MOTOR5 MOVE-SERVO EXIT
        SELECT 20   2 MOTOR6 MOVE-SERVO EXIT
        SELECT 21  -2 MOTOR6 MOVE-SERVO EXIT
        SELECT 12  NULL-SERVOS          EXIT
    END-SELECT
    ;

$B7 BIT-SFR LED                         \ Toon toets aanslag op ledje

: SERVO-CONTROL ( -- )                  \ Bestuur servo's via RC5 afstandbediening
    SETUP                               \ Initialiseer Forth,
    SERVO-SETUP                         \ de servo hardware
    RC-SETUP                            \ en de RC5 decoder
    NULL-SERVOS                         \ Servos in start positie
    CLEAR LED                           \ Startup signaal, led aan
    100 10MS                            \ Wacht een seconde
    BEGIN
        50 MS                           \ Wacht even
        TOGGLE LED                      \ Zet led uit
        RC-KEY                          \ Wacht op commando
        MOVE-A-SERVO                    \ Verplaats een servo
        TOGGLE LED                      \ Zet led aan
    AGAIN
    ;

' SERVO-CONTROL   RESET-VEC SET-VECTOR  \ Installeer SERVO programma

\ Einde
