\ ==============================================================
\ [TITLE]     AT89Cx051 ByteForth arithmetic library. 
\ [FILE]      ARITH.FRT
\ [CATEGORY]  Library ( 394 bytes )
\ [VSN]       1.00 
\ [CREATED]   April 9, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] April 10, 1995, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   Arithmetic routines for ByteForth
\ ==============================================================

ATOM UM/MOD

\G n3 is de kleinste van het paar n1 en n2.
: MIN               ( n1 n2 -- n3 )     \ Arith
    2DUP < IF  DROP EXIT  THEN  NIP
    ;

\G n3 is de grootste van het paar n1 en n2.
: MAX               ( n1 n2 -- n3 )     \ Arith
    2DUP > IF  DROP EXIT  THEN  NIP
    ;

\G d is het signed produkt van n1 maal n2.
: M*                ( n1 n2 -- d )      \ Arith "m-star"
    2DUP XOR >R
    ABS SWAP ABS UM*
    R> 0< IF  DNEGATE  THEN
    ;

\G Deel d door n1, n2 is de rest en n3 is het 'floored' quotient.
: FM/MOD            ( d n1 -- n2 n3 )   \ Arith "f-m-slash-mod"
    DUP >R ABS  OVER 0<
    IF   TUCK + SWAP
    THEN UM/MOD R@ 0<
    IF   NEGATE OVER
         IF   SWAP R@ + SWAP 1-
         THEN
    THEN R> DROP
    ;

\G Deel d door n1, n2 is de rest en n3 het 'symetric' quotient.
: SM/REM            ( d n1 -- n2 n3 )   \ Arith "s-m-slash-rem"
    2DUP XOR >R  OVER >R
    ABS >R  DABS  R> UM/MOD
    SWAP R> ?NEGATE
    SWAP R> ?NEGATE
    ;

\G Vermenigvuldig n1 met n2 met het dubbel tussenresultaat. Deel
\G de dubbel daarna door n3, n4 is de rest en n5 is het quotient.
: */MOD             ( n1 n2 n3 -- n4 n5 ) \ Arith "star-slash-mod"
    >R  M*  R>  SM/REM
    ;

\G Vermenigvuldig n1 met n2 met het dubbel tussenresultaat.
\G Deel de dubbel daarna door n3, n4 is het quotient.
: */                ( n1 n2 n3 -- n4 )  \ Arith "star-slash"
    */MOD NIP
    ;

\G Deel n1 door n2, n3 is de rest en n4 het quotient.
: /MOD              ( n1 n2 -- n3 n4 )  \ Arith "slash-mod"
    >R  S>D  R> SM/REM
    ;

\G Deel n1 door n2, n3 is het quotient.
: /                 ( n1 n2 -- n3 )     \ Arith "slash"
    /MOD NIP
    ;

\G Deel n1 door n2, n3 is de rest.
: MOD               ( n1 n2 -- n3 )     \ Arith
    /MOD DROP
    ;

\ End
