\ ==============================================================
\ [TITLE]     89Cx051 ByteForth BAMBOE target library. 
\ [FILE]      BAMBOE.FRT
\ [CATEGORY]  Library 41 bytes
\ [VSN]       1.00 
\ [CREATED]   January 16, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] December 5, 1996, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994-1996
\ [COMPILE]   ByteForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   8 bits I/O, Bamboe using 5 io-port bits on port 4
\             Example of use of machine code compared to high level
\ ==============================================================

1 [IF]

$08 =: #BITS

\ Connections to port P3
\
\ Bit 0 = Free
\ Bit 1 = Free
\ Bit 2 = Free
\ Bit 3 = Load input
\ Bit 4 = Output ready
\ Bit 5 = Clock
\ Bit 6 = Data to Bamboe outputs
\ Bit 7 = Data from Bamboe inputs

\G Stuur de data x naar de bamboe uitgangen.
CODE BAMBOE!  ( x -- )                  \ Bamboe "bamboe-store" At 12 MHz it takes 56 microsec.
    A: @R0 MOV,                         \ Get byte
    R0: INC,                            \ Pop byte
    P3: .3 SETB,                        \ Init. (don't sample input bits).
    R2: #BITS # MOV,                    \ A Bamboe has #BITS bits
    DO,
        A: RLC,                         \ Shift to left thru carry
        P3: .6 C: MOV,                  \ Set one bit ready
        P3: .5 CLR,                     \ Send data, make clockbit low 
        P3: .5 SETB,                    \ Set data, make clockbit high
    R2: LOOP,                           \ Do this #BITS times
    P3: .4 SETB,                        \ Ready, shiftreg. to outputs
    P3: .4 CLR,                         \ And disconnect them again 
    RET,                                \ End
END-CODE

\G Lees de data x van de bamboe ingangen.
CODE BAMBOE@    ( -- x )                \ Bamboe "bamboe-fetch" At 12 MHz it takes 57 microsec.
    P3: .3 CLR,                         \ Load (sample) inputs to shiftregister
    P3: .7 SETB,                        \ Init. (inputbit activated).
    P3: .3 SETB,                        \ Disconnect inputs.
    R2: #BITS # MOV,                    \ Bamboe has #BITS bits
    DO,
        C: P3: .7 MOV,                  \ Read one data bit
        P3: .5 CLR,                     \ Get new databit, make clockbit low
        P3: .5 SETB,                    \ Make clockbit high
        A: RRC,                         \ Shift received bit in it to right
    R2: LOOP,                           \ Do this #BITS times
    R0: DEC,                            \ Make stackspace
    A: CPL,                             \ Complement all bits
    @R0 A: MOV,                         \ Set at lowbyte position
    RET,                                \ End
END-CODE

[ELSE]

 ASSEMBLER
 P3: SFR POORT                         \ Port 3 of 89Cx051
 WORK

 $00 CONSTANT LEES-INGANG              \ Bit patterns for wanted functions
 $18 CONSTANT UITGANG-KLAAR 
 $80 CONSTANT DATA-IN
 $40 CONSTANT DATA-UIT
 $28 CONSTANT KLOK
 $08 CONSTANT NUL

 $08 CONSTANT #BITS                     \ Number of output bits
 
 : SCHRIJF-BIT  ( x -- )                \ Write one bit
    DUP TO POORT  KLOK OR TO POORT 
    ;
 
 : BAMBOE!      ( x -- )                \ Write #BITS bits
    0 #BITS 1- 
    DO
        DUP  1 I LSHIFT  AND 0= 0=
        DATA-UIT AND  NUL OR  SCHRIJF-BIT
    -1 +LOOP
    DROP  UITGANG-KLAAR TO POORT 
    ;
 
 : LEES         ( -- x )                \ Read input port
    DATA-IN NUL OR TO POORT  POORT 
    ;

 : LEES-BIT     ( -- x )                \ Leave $80 when input is zero
    LEES DATA-IN AND 0=   $80 AND 
    ;
 
 : BAMBOE@      ( -- x )                \ Read #BITS bamboe input bits
    LEES-INGANG TO POORT  NUL TO POORT
    0 #BITS 0
    DO  1 RSHIFT 
        LEES-BIT  OR
        NUL TO POORT  KLOK TO POORT
    LOOP 
    ;

[THEN]

\ End
