\ ==============================================================
\ [TITLE]     ByteForth double number routines
\ [FILE]      DOUBLE.FRT
\ [CATEGORY]  Library
\ [VSN]       1.01
\ [CREATED]   November 8, 1996, Willem Ouwerkerk
\ [LASTCHNGD] December 12, 1997, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1996-1997
\ [USES]      AT89Cx051
\ [PURPOSE]   16 bit arithmetic and comparision routines
\ ==============================================================

\ To be extended in the future with: DUM/MOD and D/ etc.

ATOM 2SWAP      ATOM 2OVER      ATOM DABS

\G Vermenigvuldig +dx1 en +dx2 met als resultaat ud.
CODE DU*            ( +dx1 +dx2 -- ud ) \ Double "du-star"
    [R1] @R0 MOV,                       \ H2
    R0: INC,
    [R2] @R0 MOV,                       \ L2
    R0: INC,
    [R3] @R0 MOV,                       \ H1
    R0: INC,
    [R4] @R0 MOV,                       \ L1
    A: R2: MOV,                         \ Get L2
    B: R4: MOV,                         \ Get L1
    AB: MUL,                            \ L1*L2
    @R0 A: MOV,                         \ Low part ready
    R0: DEC,                            \ to high
    A: R2: MOV,                         \ Get L2
    R2: B: MOV,                         \ Save High part of L1*L2
    B: R3: MOV,                         \ Get H1
    AB: MUL,                            \ H1*L2
    A: R2: ADD,                         \ (H1*L2)+H(L1*L2)
    R2: A: MOV,                         \ Save result of previous
    A: R4: MOV,                         \ Get L1
    B: R1: MOV,                         \ Get H2
    AB: MUL,                            \ L1*H2
    A: R2: ADD,                         \ L(L1*H2)+result ...
    @R0 A: MOV,
    RET,
END-CODE

\G Vermenigvuldig d1 en d2 met het signed resultaat d3.
: D*                ( d1 d2 -- d3 )     \ Double "d-star"
    2 PICK
    OVER XOR >R
    DABS  2SWAP  DABS  DU*
    R> 0< IF  DNEGATE  THEN
    ;

\G De vlag is true als d1 kleiner is dan d2.
: D<                ( d1 d2 -- flag )   \ Double "d-less"
    ROT 2DUP =
    IF    -ROT D- 0<
    ELSE  SWAP < NIP
    THEN  NIP
    ;

\G De vlag is true als d1 groter is dan d2.
: D>                ( d1 d2 -- flag )   \ Double "d-greater"
    2SWAP  D<
    ;

\G De vlag is true als d een positief getal groter dan nul is.
: D0>               ( d -- flag )       \ Double "d-zero-greater"
    NIP 0>                              \ Debugged 15-06-99 *WO*
    ;

\G De vlag is true als dx kleiner is dan nul.
: D0<               ( d -- flag )       \ Double "d-zero-less"
    NIP 0<
    ;

\G De vlag is true als dx gelijk is aan nul.
: D0=               ( dx -- flag )      \ Double "d-zero-equals"
    OR 0=
    ;

\G De vlag is true als dx1 is gelijk is aan xd2.
: D=                ( dx1 dx2 -- flag ) \ Double "d-equals"
    D-  OR 0=
    ;

\G De vlag is true als dx1 ongelijk is aan dx2.
: D<>               ( dx1 dx2 -- flag ) \ Double "d-not-equal"
    D-  OR 0= INVERT
    ;

\G High level toegang tot de carry vlag, de flag is true als de
\G carry gezet is.
$D7 BIT-SFR CARRY   ( -- flag )         \ Double

\G De vlag is true als du1 kleiner is dan du2.
: DU<               ( du1 du2 -- flag ) \ Double "d-u-less"
    D-  OR 0= IF  FALSE EXIT  THEN  CARRY
    ;

\G De vlag is true als du1 groter is dan du2.
: DU>               ( du1 du2 -- flag ) \ Double "d-u-greater"
    2SWAP DU<
    ;

\G d3 is de grotere van d1 en d2.
: DMAX              ( d1 d2 -- d3 )     \ Double "d-max"
    2OVER 2OVER D< IF  2SWAP  THEN  2DROP
    ;

\G d3 is de kleinere van d1 en d2.
: DMIN              ( d1 d2 -- d3 )     \ Double "d-min"
    2OVER 2OVER D> IF  2SWAP  THEN  2DROP
    ;

\G du3 is de grotere van du1 en du2.
: DUMAX             ( du1 du2 -- du3 )  \ Double "d-u-max"
    2OVER 2OVER DU< IF  2SWAP  THEN  2DROP
    ;

\G du3 is de kleinere van du1 en du2.
: DUMIN             ( ud1 ud2 -- ud3 )  \ Double "d-u-max"
    2OVER 2OVER DU> IF  2SWAP  THEN  2DROP
    ;

2VARIABLE ANUMBER  VARIABLE SQRT        \ Work space

\G Bereken de wortel van du, met als resultaat u.
: DUSQRT            ( du -- u )         \ Double "d-u-square-root" 
    TO ANUMBER  CLEAR SQRT  $80
    8 FOR
        DUP FROM SQRT +  DUP UM*  
        FROM ANUMBER DU> 0=
        IF  DUP +TO SQRT  THEN  U2/
    NEXT
    DROP  FROM SQRT
    ;

\ Einde
