\ PCF8583 en PCF8593, 24 uurs klok gebruik: 555 bytes (Prijs fl 11,75)
\ inclusief de I2C-PRIM.FRT bibliotheek file (131 bytes).
\
\ 20 30 10    ZET-KLOK    is: 10 uur 30 min 20 sec
\ 29 01 1998. ZET-DATUM   is: 29-01-1998

\ Heeft I2C-PRIM.FRT                    \ I2C primitieven nodig uit LIB

\ PCF8583 primitieven

$A0 CONSTANT KLOK-ID                    \ PCF8583 identificatie adres

\G Zet data x op adres addr van de PCF8583.
: !KLOK         ( x addr -- )           \ I2C-8583  Zet x in adres addr
    START-BIT
    KLOK-ID BYTE-UIT                    \ Stuur chip adres schrijven
    BYTE-UIT                            \ Stuur adres byte en
    BYTE-UIT                            \ daarna de databyte
    STOP-BIT                            \ Klaar
    ;

\G Lees data x van adres addr uit de PCF8583.
: @KLOK         ( addr -- x )           \ I2C-8583  Lees de inhoud van adr, x
    START-BIT
    KLOK-ID BYTE-UIT                    \ Stuur chip adres schrijven
    BYTE-UIT                            \ Stuur adres
    START-BIT                           \ Geef 'repeated start' 
    KLOK-ID 1+ BYTE-UIT                 \ Stuur chip adres lezen
    BYTE-IN                             \ Lees inhoud adres 
    NACK-BIT
    STOP-BIT                            \ Klaar
    ;

\ PCF8583 BCD omzetting

: >BCD              ( byte1 -- byte2 )  \ I2C-8583  "to-bcd" Converteer binair naar BCD.
    9 0 DO
        DUP #10 -  DUP 0< IF            \ Tientallen op ?
            DROP  I  LEAVE              \ Ja, klaar gooi oude getal weg
        ELSE
            NIP                         \ Nee, ga door met nieuw getal
        THEN
    LOOP
    #16 *  +                            \ Maak BCD getal
    ;

: BCD>              ( byte1 -- byte2 )  \ I2C-8583  "from-bcd" Converteer BCD naar binair.
    SPLIT  #10 *  +
    ;

\ PCF8583 klok 

$00 CONSTANT NORMALE                    \ I2C-8583  Klok zonder wekker
$04 CONSTANT WEK                        \ I2C-8583  Klok met wekker

\G Zet klok mode van de PCF8583. Geldige modes zijn: NORMALE en WEK
: KLOK              ( mode -- )         \ I2C-8583  Zet de klokmode van PCF8583
    0 !KLOK
    ;

\G Zet de tijd.
: ZET-KLOK  ( sec min uur -- )          \ I2C-8583  Zet PCF8583 op tijd
    0  1 !KLOK  >BCD 4 !KLOK            \ Converteer naar BCD en zet weg
    >BCD 3 !KLOK  >BCD 2 !KLOK
    ;    

\G Lees de tijd.
: LEES-KLOK ( -- sec min uur )          \ I2C-8583  Lees tijd van PCF8583
    2 @KLOK BCD>  3 @KLOK BCD>          \ en zet om in binair
    4 @KLOK BCD>
    ;

2VARIABLE JAAR                          \ Jaren teller
VARIABLE #JAAR                          \ Schrikkel jaar controle

\G Zet de datum (djaar is een dubbel getal).
: ZET-DATUM  ( dag mnd djaar -- )       \ I2C-8583  Zet datum
    2DUP TO JAAR  OVER 3 AND TO #JAAR   \ Zet jaar
    DROP 6 LSHIFT >R  >BCD 6 !KLOK      \ Schaal jaarbits in en sla maand op
    >BCD R> OR  5 !KLOK                 \ Pak dag samen met jaar en sla op
    ;

\G Lees de datum (djaar is een dubbel getal).
: LEES-DATUM  ( -- dag mnd djaar )      \ I2C-8583  Geef datum
    5 @KLOK 6 RSHIFT  DUP FROM #JAAR <> \ Nieuw jaar begonnen
    IF  1 +TO JAAR  THEN  TO #JAAR      \ Ja, verhoog jaar teller
    5 @KLOK $1F AND BCD>                \ Vis dag op
    6 @KLOK $3F AND BCD>   FROM JAAR    \ Vis maanden en jaar op
    ;

\ Gebruik wekker klok functie van PCF8583

$00 CONSTANT GEEN                       \ I2C-8583  Wekker uit
$90 CONSTANT DAGELIJKSE                 \ I2C-8583  Wekker iedere dag om .. uur
$A0 CONSTANT WEEKDAG                    \ I2C-8583  Wekker iedere .. weekdag op ..
$B0 CONSTANT DATUM                      \ I2C-8583  Wekker op datum om .. 

\G Zet wekkermode, geldige modes zijn: 
\G GEEN         wekker = Geen wekker functie.
\G DAGELIJKSE   wekker = Wek op elke dag af.
\G WEEKDAG      wekker = Wek alleen op deze dag.
\G DATUM        wekker = Wek alleen op deze datum.
: WEKKER            ( mode -- )         \ I2C-8583  Selecteer wekker mode
    8 !KLOK
    ;

\G Zet wektijd.
: ZET-WEK-TIJD      ( sec min uur -- )  \ I2C-8583  Zet wekker tijd en mode
    0 9 !KLOK  >BCD 12 !KLOK            \ Tikken altijd 0, zet uren
    >BCD 11 !KLOK  >BCD 10 !KLOK        \ Zet min. en sec.
    ;

\G Lees de wektijd.
: LEES-WEKKER       ( -- sec min uur )  \ I2C-8583  Lees wektijd
    10 @KLOK BCD>  11 @KLOK BCD>        \ Lees sec, minuten
    12 @KLOK BCD>                       \ en uren
    ;

\G De vlag is true als de wekker afgelopen is.
: WEKKER?           ( -- vlag )         \ I2C-8583  Wekker af gegaan ?
      0 @KLOK  2 AND 0= 0=
      ;

\G Zet wek datum (het jaar wordt niet gebruikt).
: ZET-WEK-DATUM   ( dag mnd djaar -- )  \ I2C-8583  Zet dag en maand en jaar bits
    DROP 6 LSHIFT >R  >BCD 14 !KLOK
    >BCD R> OR  13 !KLOK
    ;

\G Initialiseer een normale 24 uurs klok zonder wekker.
: SETUP-KLOK        ( -- )              \ I2C-8583  
    NORMALE KLOK  GEEN WEKKER           \ Zet klok aan en wekker uit
    ;

\ Einde
