\ ==============================================================
\ [TITLE]     AT89C2051 ByteForth MIDI target library. 
\ [FILE]      MIDI.FRT
\ [CATEGORY]  Library 89 bytes
\ [VSN]       1.01
\ [CREATED]   June 16, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] June 16, 1997, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1997
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   MIDI I/O on target with use of timer 1 (12 MHz).
\ ==============================================================

$99 =: SBUF:                            \ Labels for 80Cxx
$99 =: TI:
$98 =: RI:
$87 =: PCON:
$89 =: TMOD:
$8D =: TH1:
$8E =: TR1:
$98 =: SCON:
$AC =: ES:

\G Initialiseer de ACIA op een MIDI baudrate van 31K25 baud.
CODE INIT-MIDI      ( -- )              \ Midi Install 31K25 midi baudrate
    TMOD: $20 # MOV,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: 1 # MOV,                       \ Select 31250 baud
    PCON: $80 # ORL,                    \ Set SMOD high
    SCON: $7A # MOV,  ( $52 )           \ Serial mode 1
    ES: CLR,                            \ Disable SIO interrupt
    RET,
END-CODE

\G Verzend de data byte via MIDI.
CODE MIDI-EMIT      ( byte -- )         \ Midi Send midi databyte
    A: @R0 MOV,
    R0: INC,
    BEGIN, TI: BS UNTIL,
    SBUF: A: MOV,
    TI: CLR,
    RET,
END-CODE

\G Verzend u bytes vanaf ROM adres $addr via MIDI.
CODE MIDI-TYPE      ( $addr u -- )      \ Midi Type midi string from ROM
    [R1] @R0 MOV,
    R0: INC,
    DPH: @R0 MOV,
    R0: INC,
    DPL: @R0 MOV,
    R0: INC,
    A: R1: MOV,
    NZR IF,
        DO,     A: CLR,
                A: @A+DPTR MOVC,
                BEGIN, TI: BS UNTIL,
                SBUF: A: MOV,
                TI: CLR,
                DPTR: INC,
        R1: LOOP,
    THEN,
    RET,
END-CODE

\G Verstuur u bytes vanaf RAM adres addr via MIDI.
CODE MIDI-RTYPE     ( addr u -- )       \ Midi Type midi string from RAM
    [R2] @R0 MOV,
    R0: INC,
    [R1] @R0 MOV,
    R0: INC,
    A: R2: MOV,
    NZR IF,
        DO,     BEGIN, TI: BS UNTIL,
                SBUF: @R1 MOV,
                TI: CLR,
                R1: INC,
        R2: LOOP,
    THEN,
    RET,
END-CODE

\G Geef true als een byte via MIDI ontvangen is, anders false.
CODE MIDI-KEY?      ( -- flag )         \ Midi  Midi data received
    A: CLR,
    RI: BS IF,
        A: DEC,
    THEN,
    R0: DEC,
    @R0 A: MOV,
    RET,
END-CODE

\G Wacht tot de data x via MIDI ontvangen is.
CODE MIDI-KEY       ( -- byte )         \ Midi Wait for midi data and push
    BEGIN, RI: BS UNTIL,
    R0: DEC,
    @R0 SBUF: MOV,
    RI: CLR,
    RET,
END-CODE

\ End
