\ ==============================================================
\ [TITLE]     AT80Cx051 ByteForth music control
\ [FILE]      MUSIC.FRT
\ [CATEGORY]  Application ( 210 bytes )
\ [VSN]       1.00 
\ [CREATED]   June 19, 1995, Willem Ouwerkerk
\ [LASTCHNGD] November, 12, 1996, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1995-1996
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   Making music with a 12 MHz xtal.
\ ==============================================================

CODE MS             ( u -- )            \ Wait u milliseconds
    A: @R0 MOV,
    R0: INC,
    NZR IF,
        R1: A: MOV,
        DO,     R2: 166 # MOV,
                DO,     ( NOP, )        \ 6 MHz
                        AB: MUL,        \ 12 MHz
                R2: LOOP,
        R1: LOOP,
    THEN,
    RET,
END-CODE            

VARIABLE NOTE-TIME                      \ Length of note timer
VARIABLE TEMPO                          \ Length of a note

CODE PERIOD         ( -- )              \ Delay 10 us to 2550 us (1-255)
    R4: [R2] MOV,
    DO,     AB: MUL,                    \ NOP, NOP, NOP, at 6 MHz
            AB: MUL,       
    R4: LOOP,
    RET,
END-CODE

CODE SQUARE         ( period -- )       \ Squarewave with a period time of
    [R2] @R0 MOV,                       \ period * 10 us
    R0: INC,
    NOTE-TIME 0 # MOV,
    BEGIN,  P1: .5 SETB,
            ' PERIOD CALL,
            P1: .5 CLR,
            ' PERIOD CALL,
            C: SETB,
            A: TEMPO MOV,
            A: NOTE-TIME SUBB,
    CS UNTIL,
    RET,
END-CODE

\G Wacht TEMPO milliseconden.
: REST              ( -- )              \ Music Silence for TEMPO millisec.
    FROM TEMPO MS ;

\G Zet TEMPO zo dat hele noten gespeeld worden.
\G Er zijn ook: 1/2 1/4 en 1/8 tempo's.
: 1/1       250 TO TEMPO ;   ( -- )     \ Music

: 1/2       125 TO TEMPO ;
: 1/4        63 TO TEMPO ;
: 1/8        31 TO TEMPO ;

\G Speel de noot A1 met de lengte van TEMPO millisec. Er zijn nog 22
\G andere noten, die samen ongeveer 2.3 octaaf vormen van A1 tot C3.
: A1        227 SQUARE ;     ( -- )     \ Music 2.3 octave

: B1        200 SQUARE ;
: C1        192 SQUARE ;
: D1        172 SQUARE ;
: E1        152 SQUARE ;
: F2        143 SQUARE ;
: FIS2      135 SQUARE ;
: G2        128 SQUARE ;
: GIS2      119 SQUARE ;
: A2        114 SQUARE ;
: AIS2      106 SQUARE ;
: B2        102 SQUARE ;
: C2         96 SQUARE ;
: CIS2       91 SQUARE ;
: D2         85 SQUARE ;
: DIS2       81 SQUARE ;
: E2         76 SQUARE ;
: F3         71 SQUARE ;
: G3         64 SQUARE ;
: A3         57 SQUARE ;
: B3         51 SQUARE ;
: C3         48 SQUARE ;

\G Initialiseer de muziek hardware.
CODE SETUP-MUSIC  ( -- )                \ Music Install timer 0
    TMOD: $11 # MOV,
    TH0: -1000. NIP # MOV,              \ -500. with a 6 MHz xtal.
    TL0: -1000. DROP # MOV,
    IE: .7 SETB,
    IE: .1 SETB,
    TCON: .4 SETB,
    RET,
END-CODE

CODE TIMER0                             \ Interrupt code
    TH0: -1000. NIP # MOV,              \ -500. with a 6 MHz xtal.
    TL0: -1000. DROP # MOV,
    NOTE-TIME INC,
    RETI,
END-CODE

DOC  Play an octave in differend tempo's

: OCTAAF                                \ Demo word
    REST F2 FIS2 G2 GIS2 A2 AIS2 B2 C2 CIS2 D2 DIS2 E2 REST
    ;

: MUSIC-DEMO    ( -- )
    SETUP
    SETUP-MUSIC
    BEGIN 
        1/8 OCTAAF
        1/4 OCTAAF
        1/2 OCTAAF
        1/1 OCTAAF
        1/2 OCTAAF
        REST A1  REST A2  REST A3
        REST C1  REST C2  REST C3
    AGAIN
    ;

MUSIC-DEMO  RESET-VEC   SET-VECTOR      \ Install startup routine

ENDDOC

TIMER0      TF0-VEC     SET-VECTOR      \ Install timer 0 interrupt

\ End
