\ ==============================================================
\ [TITLE]     AT89Cx051 ByteForth number output library. 
\ [FILE]      NUMBERS.FRT
\ [CATEGORY]  Library ( 156 bytes )
\ [VSN]       1.00 
\ [CREATED]   April 10, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] Januari 15, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1998
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   Number output to current output device
\             Needs: LCD.FRT or RS232.FRT
\ ==============================================================

\G addr is het adres van de cel die het grondtal voor de getal
\G conversie bevat, geldige waarden zijn 2 t/m 36.
VARIABLE BASE      ( -- addr )          \ Numbers Hold number conversion base

VARIABLE HLD                            \ Work addr. in number conv. buffer
$80 CONSTANT NUM                        \ Start of number conversion buffer
                                        \ Only for AT89C2051
\ $40 CONSTANT NUM                      \ Start of number conversion buffer
                                        \ Only for AT89C1051

\ Unsigned devide routine

\G Deel ud1 door het getal in BASE tot ud2 en char. Wordt gebruikt
\G om een digit te reduceren van een unsigned dubbel getal tot zijn
\G equivalent ASCII karakter.
CODE DIG-OUT ( ud1 -- ud2 char )        \ Numbers "dig-out" Divide ud1 by [BASE] to ud2 with char
      R2: ADR BASE MOV,                 \ Get base as divisor in R2
      R0: DEC,
      R1: [R0] MOV,                     \ Copy stack pointer to R1 (<ud1>-byte3)
      R6: 2 # MOV,                      \ Process 2 bytes dividend
      R3: 0 # MOV,                      \ Clear shift/work register
      DO,                               \ Start of outer divide loop
         R5: 8 # MOV,                   \ Process 8 bits per byte
         R1: INC,                       \ Position at next dividend byte
         A: @R1 MOV,                    \ Read byte as next shift register (in/out)
         DO,                            \ Start of inner divide loop
            C: SETB,                    \ Shift in a one; has to be corrected when subtract fails
            A: RLC,                     \ Shift left
            A: R3: XCH,                 \ Swap with work register
            A: RLC,                     \ Shift left into work register
            A: R2: SUBB,                \ Subtract divisor
            CS IF,                      \ Unsuccessful subtract?
               R3: DEC,                 \ Correct quotient bit 0 (Bit 0 was set by left shift)
               A: R2: ADD,              \ Restore work register
            THEN,
            A: R3: XCH,                 \ Swap work register to R3; shift reg to A
            R5: LOOP,                   \ Count bits per byte
         @R1 A: MOV,                    \ Write processed result
         R6: LOOP,                      \ Count 2 bytes
      A: R3: MOV,                       \ Retrieve remainder
      A: $F6 # ADD,                     \ Test against 10; CS if <n> >= 10
      CS IF,
         A: 7 # ADD,                    \ Correct 10 -> 'A' etc
      THEN,
      A: 58 # ADD,                      \ Convert 0..9 -> -10..-1 -> '0'..'9'
                                        \ Convert 10.. -> 0.. -> 7.. -> 'A'..
      @R0 A: MOV,                       \ Put char to stack
      RET,                              \ Done
END-CODE

\ Number conversion primitives

\G Voeg een karakter char toe aan het begin van de uitvoerstring
\G in wording. Te gebruiken tussen <# en #>.
: HOLD      DECR HLD  FROM HLD ! ;                      ( char -- )     \ Numbers

\G Als n negatief is, voeg dan een minteken toe aan de uitvoerstring
\G in wording. Te gebruiken tussen <# en #>.
: SIGN      0< IF  [CHAR] -  HOLD  THEN ;               ( n -- )        \ Numbers

\G Begin een nieuwe getal uitvoerstring.
: <#        NUM TO HLD ;                                ( -- )          \ Numbers "less-number-sign"

\G Gooi d weg en maak de uitvoerstring beschikbaar als addr u.
: #>        2DROP  FROM HLD  NUM FROM HLD - ;           ( d -- addr u ) \ Numbers "number-sign-greater"

\G Deel d1 door het getal in BASE met als resultaat d2. De rest n
\G wordt omgezet tot een karakter welke toegevoegd wordt aan het 
\G begin van de uitvoerstring. Te gebruiken tussen <# en #>.
: #         DIG-OUT HOLD ;                              ( d1 -- d2 )    \ Numbers "number-sign"

\G Converteer alle digits in d1 als beschreven bij # todat het
\G resulterende quotient d2 nul is. Te gebruiken tussen <# en #>.
: #S        BEGIN  # 2DUP OR 0= UNTIL ;                 ( d1 -- d2 )    \ Numbers "number-sign-s"

\ Number output words

\G Druk d af in een vrij formaat.
: D.        TUCK DABS <# #S  ROT SIGN #>  RTYPE SPACE ; ( d -- )        \ Numbers "d-dot"

\G Druk n af in een vrij formaat.
: .         S>D  D. ;                                   ( n -- )        \ Numbers "dot"

\G Druk u af in een vrij formaat.
: U.        0  D. ;                                     ( u -- )        \ Numbers "u-dot"

\G Zet het grondtal op 10.
: DECIMAL   #10 TO BASE ;                               ( -- )          \ Numbers

\G Zet het grondtal op 16.
: HEX       $10 TO BASE ;                               ( -- )          \ Numbers

\ End
