\ ==============================================================
\ [TITLE]     ByteForth two pseudo random generators
\ [FILE]      RANDOM.FRT
\ [CATEGORY]  Library ( 202 or 49 bytes )
\ [VSN]       1.00 
\ [CREATED]   November 12, 1996, Willem Ouwerkerk 
\ [LASTCHNGD] November 12, 1996, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1996-
\ [USES]      AT89Cx051
\ [PURPOSE]   Random generators
\ ==============================================================

2VARIABLE RND                           \ Work location for random numbers

0 [IF]                                  \ Random according Leo Brodie

ATOM DABS                               \ Import a macro

CODE DU*            ( d1 d2 -- ud )     \ Rnd "Unsigned-Double-Star"
    [R1] @R0 MOV,                       \ H2
    R0: INC,
    [R2] @R0 MOV,                       \ L2
    R0: INC,
    [R3] @R0 MOV,                       \ H1
    R0: INC,
    [R4] @R0 MOV,                       \ L1
    A: R2: MOV,                         \ Get L2
    B: R4: MOV,                         \ Get L1
    AB: MUL,                            \ L1*L2
    @R0 A: MOV,                         \ Low part ready
    R0: DEC,                            \ to high
    A: R2: MOV,                         \ Get L2
    R2: B: MOV,                         \ Save High part of L1*L2
    B: R3: MOV,                         \ Get H1
    AB: MUL,                            \ H1*L2
    A: R2: ADD,                         \ (H1*L2)+H(L1*L2)
    R2: A: MOV,                         \ Save result of previous
    A: R4: MOV,                         \ Get L1
    B: R1: MOV,                         \ Get H2
    AB: MUL,                            \ L1*H2
    A: R2: ADD,                         \ L(L1*H2)+result ...
    @R0 A: MOV,
    RET,
END-CODE

: D*                ( d1 d2 -- d3 )     \ Rnd "Double-star"
    2 PICK
    OVER XOR >R
    DABS  2SWAP  DABS  DU*
    R> 0< IF  DNEGATE  THEN
    ;

: RANDOM            ( -- ud )           \ Rnd Make random number ud
    FROM RND 31421. D*  6927. D+  2DUP TO RND
    ;

: CHOOSE            ( u1 -- u2 )        \ Rnd Make random number u2 the
    RANDOM DROP  UM* NIP                \ range of u2 is 0 to u1-1
    ;

[ELSE]                                  \ Shiftregister with XOR

CODE RANDOM         ( -- u )            \ Rnd Make random number u
    A: RND MOV,
    A: RLC,                             \ Move bit 14 to bit 15 location
    A: RND LOW XRL,                     \ Xor it with bit 7 of word
    A: RLC,                             \ Save result in carry
    A: RND LOW MOV,                     \ Shift it in at bit 0 position
    A: RLC,
    RND LOW A: MOV,
    A: RND MOV,                         \ Shift bit 7 to bit 8 position
    A: RLC,
    RND A: MOV,
    A: RND LOW MOV,                     \ We use low byte as result
    R0: DEC,                            \ Push it to the stack
    @R0 A: MOV,
    RET,
END-CODE

\G Maak het random getal u2, u2 ligt in het bereik van 0 tot u1-1.
: CHOOSE            ( u1 -- u2 )        \ Rnd Make random number u2 the
    RANDOM  UM* NIP                     \ range of u2 is 0 to u1-1
    ;

[THEN]

\G Initialiseer het random getal zaadje.
: SETUP-RANDOM      ( -- )              \ Rnd Initialise random work location
    31414. TO RND
    ;

\ End
