\ ==============================================================
\ [TITLE]     AT89C2051 ByteForth RC5 code
\ [FILE]      RC5.FRT
\ [CATEGORY]  Library ( 198 bytes )
\ [VSN]       1.20 
\ [CREATED]   May 15, 1995, Willem Ouwerkerk
\ [LASTCHNGD] November 18, 1996, Willem Ouwerkerk
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1996
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   RC5 remote control receiver ( 12 MHz )
\             Includes the user routines: RC-SETUP  MS  RC-KEY?  RC-KEY
\             And an example with the words: LED  500MS  RC-DEMO
\             It uses the macros: DROP
\             The variables for use are: RC-SYS  RC-DATA
\             Bit flags for use are: RC-KEY?
\ ==============================================================

\ 89C2051 TARGET                        \ Use while testing with demo !!

\G Geef het RC5 adres waar de decoder op reageert.
#00 CONSTANT RC-TYPE                    \ RC5 Set transmitter type
                                        \ TV = 0, CD = 20 and VCR = 5

VARIABLE RC-SYS                         \ RC5 system databyte (8 bits)
VARIABLE RC-DATA                        \ RC5 command databyte (6 bits)
VARIABLE RC#                            \ RC5 received bits count

\G De vlag is true als er een geldig RC5 commando ontvangen is.
FLAG RC-KEY?  ( -- flag )               \ RC5 "r-c-key-quistion" command received (true)

FLAG RC-BIT                             \ RC5 first half bit, for control
$B2 BIT-SFR RC-IN                       \ RC5 input bit (port 3.2)

\G Zet de hardware goed voor de RC5 decoder.
CODE RC-SETUP      ( -- )               \ RC5 Setup for RC5 receiving
( ) TMOD: $10 # ORL,                    \ Set Timer 1 in mode 1
( ) IE: .7 SETB,                        \ Global enable interupts
    TCON: .6 CLR,                       \ Stop timer 1
    IE: .3 SETB,                        \ Enable timer 1 interrupts
    TCON: .0 SETB,                      \ EX0 active on falling edges
    IE: .0 SETB,                        \ EX0 interrupt on
    ADR RC-KEY? CLR,                    \ No RC5 data received
    ADR RC-IN SETB,                     \ P3.2 is input
    RET,
END-CODE


\ RC5 remote control receiver (code for an 12 MHz xtal).

CODE RC-START      ( -- )               \ RC5 start receiving 
    PSW: PUSH,
    ADR RC-KEY? BC IF,                  \ RC5 data processed ?
        IE: .0 CLR,                     \ Yes, EX0 interrupt off
        TH1: -400. NIP # MOV,           \ Set negated 16 bits count
        TL1: -400. DROP # MOV,
        TCON: .6 SETB,                  \ Start timer 1
        RC# 1 # MOV,                    \ Half bit received
        ADR RC-BIT SETB,                \ First half bit is high!
    THEN,
    PSW: POP,
    RETI,
END-CODE

CODE SAVE-RC-BIT    ( -- )              \ Save received bit
    A: 16 # ANL, 
    ZER IF,                             \ Is it a system bit ?
        A: RC-SYS MOV,                  \ Yes
        A: RLC,
        RC-SYS A: MOV,
    ELSE,                               \ No, its a data bit
        A: RC-DATA MOV,
        A: RLC,
        RC-DATA A: MOV,
    THEN,
    RC# INC,                            \ Increase bit counter
    RET,
END-CODE

CODE READ-BIT       ( -- )              \ RC5 read bit routine
    PSW: PUSH,
    ACC: PUSH,
    TH1: -880. NIP # MOV,               \ Set negated 16 bits count
    TL1: -880. DROP # MOV,

    C: ADR RC-IN MOV,                   \ Read next RC5 bit
    A: RC# MOV,
    ACC: .0 BS IF,                      \ Collect every second bit ?
        ' SAVE-RC-BIT CALL,             \ Save a bit
        A: CLR,                         \ Check bit pattern !
        C: ADR RC-BIT MOV,
        A: RLC,
        C: ADR RC-IN MOV,
        A: 0 # ADDC,
        A: 1 # EQL IF,                  \ Valid pattern 0/1 of 1/0 received ?

            A: RC# MOV,                 \ End of bitstream
            A: $1C # EQL IF,            \ 14 bits received ?
                RC-SYS $FF # XRL,       \ Invert system bits
                RC-DATA $FF # XRL,      \ Invert data bits
                RC-DATA $3F # ANL,      \ Leave only lower 6 data bits
                A: RC-SYS MOV,
                A: $1F # ANL,           \ Leave transmitter ID
                A: RC-TYPE # EQL IF,    \ Is it a .... remote control ?
                    ADR RC-KEY? SETB,   \ Yes, new data word received !
                    IE: .0 SETB,        \ EX0 interrupt on
                    TCON: .6 CLR,       \ Stop timer 1
                THEN,
            THEN,
        ELSE,
            IE: .0 SETB,                \ EX0 interrupt on
            TCON: .6 CLR,               \ Stop timer 1
        THEN,
    ELSE,
        ADR RC-BIT C: MOV,              \ Save half bit value
        RC# INC,                        \ Increase bit count
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

CODE MS             ( u -- )            \ Wait u milliseconds
    [R1] @R0 MOV,
    R0: INC,
    DO,     R2: 166 # MOV,
            DO,     AB: MUL,
            R2: LOOP,
    R1: LOOP,
    RET,
END-CODE            

\G Wacht op een geldige RC5 toets, x is een code uit de gedefineerde
\G RC5 commandocode set. Zie ook de RC5 documentatie.
: RC-KEY        ( -- x )                \ RC5 "r-c-key" Get command databyte
    BEGIN  RC-KEY? UNTIL                \ Wait
    FROM RC-DATA                        \ Push command
    CLEAR RC-KEY?                       \ Make way
    ;

\ Install interrupt routines and demo

' READ-BIT  TF1-VEC     SET-VECTOR      \ Install timer 1 interrupt
' RC-START  IE0-VEC     SET-VECTOR      \ Install ext. interrupt 0

DOC  A small RC5-demo, show RC5 codes

$B0 BIT-SFR LED                         \ A led at P3.0
$90 SFR P1                              \ Show RC5 data

: 500MS             ( -- )              \ Wait half a second
    250 MS  250 MS
    ;

: RC-DEMO           ( -- )
    SETUP                               \ Basic setup
    RC-SETUP                            \ RC5 setup
    CLEAR LED  500MS                    \ Demo on sign
    BEGIN
        TOGGLE LED                      \ Switch led on/off
        RC-KEY  INVERT TO P1            \ Show RC5-command
( )     500MS 500MS  500MS 500MS        \ Wait 2 seconds 
( )     FROM RC-SYS  TO P1              \ Show system byte
    AGAIN
    ;

' RC-DEMO   RESET-VEC   SET-VECTOR      \ Install startup routine

ENDDOC

\ End
