\ ==============================================================
\ [TITLE]     ByteForth error recovery routines
\ [FILE]      RECOVERY.FRT
\ [CATEGORY]  Programming tools
\ [VSN]       1.00 
\ [CREATED]   November 7, 1996, Willem Ouwerkerk 
\ [LASTCHNGD] November 8, 1996, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1996-
\ [USES]      ByteForth Cross Compiler
\ [PURPOSE]   Multi level return mechanism
\ ==============================================================

ATOM EXECUTE                            \ Import execute routine (8)

\ Multi level error recovery and execute.

\ Push an exception frame to the return stack and then execute 
\ the execution token xt (as with execute). Control will resume
\ after CATCH when a trow is executed during execution of xt.
\ After normal execution CATCH will return with a zero on the
\ stack, above whatever returned by xt and the exception frame 
\ is removed. See THROW for the remainder of the description (16).
CODE CATCH          ( i*x xt -- j*x 0 | j*x n )  \ Recovery
    [R0] PUSH,                          \ Save datastack
    ERROR-HANDLER PUSH,                 \ Push old handler
    ERROR-HANDLER SP: MOV,              \ Save new error handler
    ' EXECUTE CALL,                     \ Execute xt
    ERROR-HANDLER POP,
    ACC: POP,                           \ Remove DSP
    R0: DEC,                            \ Push zero to stack
    @R0 0 # MOV,
    RET,
END-CODE

\ Pop the exception frame from the return stack and resume 
\ execution after CATCH. The data stack is restored to the
\ state prior CATCH and an error number is placed on top of 
\ that (14).
CODE THROW          ( k*x n -- k*x || i*x n )  \ Recovery
    A: @R0 MOV,                         \ Get error number
    R0: INC,                            \ Pop error number
    NZR IF,
        SP: ERROR-HANDLER MOV,          \ Restore return stack
        ERROR-HANDLER POP,              \ Restore error handler
        [R0] POP,                       \ Restore datastack
        R0: INC,                        \ Pop half of xt
        @R0 A: MOV,                     \ Move error number on top
    THEN,
    RET,
END-CODE

\ End
