\ ==============================================================
\ [TITLE]     AT89Cx051 ByteForth RS232 host & target library. 
\ [FILE]      RS232.FRT
\ [CATEGORY]  Library 148 bytes
\ [VSN]       1.01 
\ [CREATED]   April 9, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] June 16, 1997, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1997
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   RS232 I/O on host & target with use of timer 1.
\ ==============================================================

$99 CONSTANT TI:                        \ Bit labels for 80Cxxxx
$98 CONSTANT RI:
$8E CONSTANT TR1:
$AC CONSTANT ES:

6 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 6 MHz christal

\G Initialiseer de ACIA op een RS232 baudrate van 2400 baud.
CODE RS232-SETUP                        \ RS232
    TMOD: $20 # ORL,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: 243 # MOV,                     \ Select 1200/2400 baud
    PCON: $80 # ORL,                    \ At start 2400 baud
\   PCON: $7F # ANL,                    \ At start 1200 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    ES: CLR,                            \ Disable SIO interrupt
    RET,
END-CODE

[THEN]

12 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 12 MHz christal, for
\ a 6 MHz christal the baudrates are 1200 or 2400 baud.

CODE RS232-SETUP
    TMOD: $20 # ORL,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: 243 # MOV,                     \ Select 2400/4800 baud
\   PCON: $80 # ORL,                    \ At start 4800 baud
    PCON: $7F # ANL,                    \ At start 2400 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    ES: CLR,                            \ Disable SIO interrupt
    RET,
END-CODE

[THEN]

11 CRYSTAL? [IF]

\ All baudrates mentioned are for a 11.059 MHz christal.

CODE RS232-SETUP
    TMOD: $20 # ORL,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
\   TH1: $FD # MOV,                     \ Select 9600/19K2 baud
\   TH1: $F4 # MOV,                     \ Select 2400/4800 baud
    TH1: $E8 # MOV,                     \ Select 1200/2400 baud
    PCON: $80 # ORL,                    \ At start 2400 baud
\   PCON: $7F # ANL,                    \ At start 1200 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    ES: CLR,                            \ Disable SIO interrupt
    RET,
END-CODE

[THEN]

\G Verzend het karakter char. via de RS232.
CODE RS232-EMIT     ( char -- )         \ RS232 Char to RS232
    A: @R0 MOV,
    R0: INC,
    BEGIN, TI: BS UNTIL,
    SBUF: A: MOV,
    TI: CLR,
    RET,
END-CODE

\G Verzend u karakters vanaf ROM adres $addr via de RS232.
CODE RS232-TYPE     ( $addr u -- )      \ RS232 Rom string to RS232
    [R1] @R0 MOV,
    R0: INC,
    DPH: @R0 MOV,
    R0: INC,
    DPL: @R0 MOV,
    R0: INC,
    A: R1: MOV,
    NZR IF,
        DO,     A: CLR,
                A: @A+DPTR MOVC,
                BEGIN, TI: BS UNTIL,
                SBUF: A: MOV,
                TI: CLR,
                DPTR: INC,
        R1: LOOP,
    THEN,
    RET,
END-CODE

\G Verstuur u karakters vanaf RAM adres addr via de RS232.
CODE RS232-RTYPE    ( addr u -- )       \ RS232 Ram string to RS232
    [R2] @R0 MOV,
    R0: INC,
    [R1] @R0 MOV,
    R0: INC,
    A: R2: MOV,
    NZR IF,
        DO,     BEGIN, TI: BS UNTIL,
                SBUF: @R1 MOV,
                TI: CLR,
                R1: INC,
        R2: LOOP,
    THEN,
    RET,
END-CODE

\G Geef true als een karakter via de RS232 ontvangen is, anders false.
RI: BIT-SFR RS232-KEY?  ( -- flag )     \ RS232 Char present from RS232

\G Wacht tot een karakter char via RS232 ontvangen is.
CODE (RS232-KEY         ( -- char )     \ RS232 Char present but not eaten
    BEGIN, RI: BS UNTIL,
    R0: DEC,
    @R0 SBUF: MOV,
    RET,
END-CODE

\G Wacht tot een karakter char via RS232 ontvangen is.
CODE RS232-KEY      ( -- char )         \ RS232 Read char from RS232
    BEGIN, RI: BS UNTIL,
    R0: DEC,
    @R0 SBUF: MOV,
    RI: CLR,
    RET,
END-CODE

: KEY?      RS232-KEY? ;                        ( -- flag )
: (KEY      (RS232-KEY ;                        ( -- char )
: KEY       RS232-KEY ;                         ( -- char )
: EMIT      RS232-EMIT ;                        ( char -- )
: TYPE      RS232-TYPE ;                        ( $addr u -- )
: RTYPE     RS232-RTYPE ;                       ( addr u -- )
: CR        ^M RS232-EMIT  ^J RS232-EMIT ;      ( -- )
: SPACE     BL RS232-EMIT ;                     ( -- )
: SPACES    ?DUP IF  FOR  SPACE  NEXT  THEN ;   ( u -- )
: PAGE      ^L RS232-EMIT ;                     ( -- )
: BS        ^H RS232-EMIT SPACE ^H RS232-EMIT ; ( -- )

\ End
