\ ==============================================================
\ [TITLE]     AT89Cx051 ByteForth TRACER target library. 
\ [FILE]      TRACER1.FRT
\ [CATEGORY]  Library 389 bytes
\ [VSN]       1.00 
\ [CREATED]   October 10, 1997, Willem Ouwerkerk 
\ [LASTCHNGD] November 19, 1997, Willem Ouwerkerk
\ [COPYRIGHT] De Elektrise Schaar (C) 1997-
\ [USES]      ByteForth cross compiler
\ [PURPOSE]   Tracer through RS232 for target with use of timer 1.
\ ==============================================================
\ [TITLE]     AT89Cx051 ByteForth RS232 library. 
\ ==============================================================

$99 =: TI:                              \ Bit labels for 80Cx051
$98 =: RI:
$8E =: TR1:

6 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 6 MHz christal

CODE RS232-SETUP
    TMOD: $20 # MOV,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: 243 # MOV,                     \ Select 1200/2400 baud
    PCON: $80 # ORL,                    \ At start 2400 baud
\   PCON: $7F # ANL,                    \ At start 1200 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    RET,
END-CODE

[THEN]

12 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 12 MHz christal, for
\ a 6 MHz christal the baudrates are 1200 or 2400 baud.

CODE RS232-SETUP
    TMOD: $20 # MOV,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
    TH1: 243 # MOV,                     \ Select 2400/4800 baud
\   PCON: $80 # ORL,                    \ At start 4800 baud
    PCON: $7F # ANL,                    \ At start 2400 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    RET,
END-CODE

[THEN]

11 CRYSTAL? [IF]

\ All baudrates mentioned are for a 11.059 MHz christal.

CODE RS232-SETUP
    TMOD: $20 # MOV,                    \ Timer 1 mode 2 = 8 bit autoreload
    TR1: SETB,                          \ Run timer 1
\   TH1: $FD # MOV,                     \ Select 9600/19K2 baud
\   TH1: $F4 # MOV,                     \ Select 2400/4800 baud
    TH1: $E8 # MOV,                     \ Select 1200/2400 baud
    PCON: $80 # ORL,                    \ At start 2400 baud
\   PCON: $7F # ANL,                    \ At start 1200 baud
    SCON: $52 # MOV,                    \ Serial mode 1, enable reception
                                        \ Set transmit interrupt flag.
    RET,
END-CODE

[THEN]

CODE EMIT           ( char -- )         \ RS232 Char to RS232
    A: @R0 MOV,
    R0: INC,
    BEGIN, TI: BS UNTIL,
    SBUF: A: MOV,
    TI: CLR,
    RET,
END-CODE

CODE TYPE           ( $addr u -- )      \ RS232 Rom string to RS232
    [R1] @R0 MOV,
    R0: INC,
    DPH: @R0 MOV,
    R0: INC,
    DPL: @R0 MOV,
    R0: INC,
    A: R1: MOV,
    NZR IF,
        DO,     A: CLR,
                A: @A+DPTR MOVC,
                BEGIN, TI: BS UNTIL,
                SBUF: A: MOV,
                TI: CLR,
                DPTR: INC,
        R1: LOOP,
    THEN,
    RET,
END-CODE

CODE RTYPE          ( addr u -- )       \ RS232 Ram string to RS232
    [R2] @R0 MOV,
    R0: INC,
    [R1] @R0 MOV,
    R0: INC,
    A: R2: MOV,
    NZR IF,
        DO,     BEGIN, TI: BS UNTIL,
                SBUF: @R1 MOV,
                TI: CLR,
                R1: INC,
        R2: LOOP,
    THEN,
    RET,
END-CODE

CODE KEY            ( -- char )         \ RS232 Read char from RS232
    BEGIN, RI: BS UNTIL,
    R0: DEC,
    @R0 SBUF: MOV,
    RI: CLR,
    RET,
END-CODE

: CR        ^M EMIT  ^J EMIT ;                  ( -- )
: SPACE     BL EMIT ;                           ( -- )

\ ==============================================================
\ [TITLE]     AT89Cx051 ByteForth number output library. 
\ ==============================================================

ATOM INLINE$

VARIABLE BASE                           \ Hold number conversion base
VARIABLE HLD                            \ Work addr. in number conv. buffer
$80 CONSTANT NUM                        \ Start of number conversion buffer
                                        \ Only for AT89C2051
\ $40 CONSTANT NUM                      \ Start of number conversion buffer
                                        \ Only for AT89C1051


\ Unsigned devide routine

CODE DIG-OUT ( ud1 -- ud2 char )        \ Numbers "dig-out" Divide ud1 by [BASE] to ud2 with char
      R2: ADR BASE MOV,                 \ Get base as divisor in R2
      R0: DEC,
      R1: [R0] MOV,                     \ Copy stack pointer to R1 (<ud1>-byte3)
      R6: 2 # MOV,                      \ Process 2 bytes dividend
      R3: 0 # MOV,                      \ Clear shift/work register
      DO,                               \ Start of outer divide loop
         R5: 8 # MOV,                   \ Process 8 bits per byte
         R1: INC,                       \ Position at next dividend byte
         A: @R1 MOV,                    \ Read byte as next shift register (in/out)
         DO,                            \ Start of inner divide loop
            C: SETB,                    \ Shift in a one; has to be corrected when subtract fails
            A: RLC,                     \ Shift left
            A: R3: XCH,                 \ Swap with work register
            A: RLC,                     \ Shift left into work register
            A: R2: SUBB,                \ Subtract divisor
            CS IF,                      \ Unsuccessful subtract?
               R3: DEC,                 \ Correct quotient bit 0 (Bit 0 was set by left shift)
               A: R2: ADD,              \ Restore work register
            THEN,
            A: R3: XCH,                 \ Swap work register to R3; shift reg to A
            R5: LOOP,                   \ Count bits per byte
         @R1 A: MOV,                    \ Write processed result
         R6: LOOP,                      \ Count 2 bytes
      A: R3: MOV,                       \ Retrieve remainder
      A: $F6 # ADD,                     \ Test against 10; CS if <n> >= 10
      CS IF,
         A: 7 # ADD,                    \ Correct 10 -> 'A' etc
      THEN,
      A: 58 # ADD,                      \ Convert 0..9 -> -10..-1 -> '0'..'9'
                                        \ Convert 10.. -> 0.. -> 7.. -> 'A'..
      @R0 A: MOV,                       \ Put char to stack
      RET,                              \ Done
END-CODE

\ Number conversion primitives

: HOLD      DECR HLD  FROM HLD ! ;                      ( char -- )
: SIGN      0< IF  [CHAR] -  HOLD  THEN ;               ( n -- )
: <#        NUM TO HLD ;                                ( -- )
: #>        2DROP  FROM HLD  NUM FROM HLD - ;           ( d -- addr u )
: #         DIG-OUT HOLD ;                              ( d1 -- d2 )
: #S        BEGIN  # 2DUP OR 0= UNTIL ;                 ( d1 -- d2 )

\ Number output words

: U.        0 <# #S  #>  RTYPE SPACE ;                  ( d -- )

: DECIMAL   #10 TO BASE ;                               ( -- )


\ Tracer routines

ASSEMBLER
ACC: SFR ACCU
PSW: SFR PSW
SP:  SFR SP
WORK

\G Toon de belangrijkste registers van de AT89Cx051.
: .REGISTERS    ( -- )                  \ Tracer "dot-registers" Show most important registers
    SP  PSW  ACCU  CR                   \ Read registers
    ." Accu: "      U.                  \ Accumulator
    ." Statusreg: " U.                  \ Program Status Word
    ." Stackp: "    U.                  \ Hardware Stackpointer
    ;

\G Copieer en toon alle gegevens op de datastack.
: .S            ( -- )                  \ Tracer "dot-s" Show stack picture
   ." ( "
    DEPTH IF
        DEPTH FOR
            I' 1- PICK U.
        NEXT
   THEN
   ." ) "
   ;

\G Zet een breakpunt, toon de registers en druk de stack af,
\G wacht daarna op een toetsaanslag.
: BREAKPOINT    ( -- )                  \ Tracer Show tracing info
    .REGISTERS                          \ Importand registers
    CR .S                               \ the datastack
    KEY DROP                            \ and wait for a key
    ;

\G Configureer de tracer en zet een breakpunt.
: TRACER-SETUP  ( -- )                  \ Tracer Setup for higlevel tracer
    RS232-SETUP
    DECIMAL
    BREAKPOINT
    ;

\ End
