\ ----------------------------------------------------------------------
\ LANGUAGE    : ByteForth
\ PROJECT     : I2C temperatuur driver network
\ DESCRIPTION : Measuring temperature with LM75CIM
\ CATEGORY    : Application, size: 415 bytes.
\ AUTHOR      : Willem Ouwerkerk, November 25, 1999
\ LAST CHANGE : Willem Ouwerkerk, November 26, 1999
\ ----------------------------------------------------------------------

89C2051
NEEDS TARGET.FRT
NEEDS I2C-PRIM.FRT

$90 CONSTANT LM75-ID                \ LM75 basis adres

: SELECT-CHIP   ( chip -- dev-id )  \ Max. 8 chips
    7 AND 2*  LM75-ID  +
    ;

: REGISTER      ( reg chip -- )     \ Selecteer LM75 register
    START-BIT
    SELECT-CHIP  BYTE-UIT
    3 AND BYTE-UIT
    STOP-BIT
    ;

\ Temperatuur wordt gegeven in halve graden per bit, waar 0
\ nul graden is, en het bereik loopt van -55 tot 125 graden.
: TEMPERATUUR   ( chip -- dx )      \ Lees temperatuur als
    START-BIT                       \ 9 bits getal
    SELECT-CHIP  1 OR  BYTE-UIT
    BYTE-IN  ACK-BIT  >R            \ msb
    BYTE-IN  NACK-BIT  STOP-BIT     \ lsb
    7 RSHIFT  R@ 2*  OR             \ Zet temperatuur om naar
    R> 7 LSHIFT                     \ een Forth dubbel getal
    ;

: CONFIGURATIE  ( x chip -- )       \ Zet LM75 configuratie
    >R  1 R@ REGISTER               \ Selecteer config. reg.
    START-BIT
    R@ SELECT-CHIP  BYTE-UIT
    BYTE-UIT  STOP-BIT              \ Zet config. register
    0 R> REGISTER                   \ Selecteer temp. reg.
    ;

: >TEMPERATUUR  ( +d -- dx )        \ Converteer Forth dubbel
    7 LSHIFT  OVER 1 RSHIFT  OR     \ naar LM75 temperatuur
    SWAP  7 LSHIFT  SWAP
    ;

: ONDERGRENS    ( dx chip -- )
    >R  2 R@ REGISTER               \ Selecteer THYST reg.
    START-BIT
    R@ SELECT-CHIP  BYTE-UIT
    >TEMPERATUUR  BYTE-UIT          \ Zet thermostaat
    BYTE-UIT  STOP-BIT              \ ondergrens
    0 R> REGISTER                   \ Selecteer temp. reg.
    ;

: BOVENGRENS    ( dx chip -- )
    >R  3 R@ REGISTER               \ Selecteer TOS reg.
    START-BIT
    R@ SELECT-CHIP  BYTE-UIT
    >TEMPERATUUR  BYTE-UIT          \ Zet thermostaat
    BYTE-UIT  STOP-BIT              \ bovengrens
    0 R> REGISTER                   \ Selecteer temp. reg.
    ;

                               ( End )
